"""
Parsers for RabbitMQ
====================

Parsers included in this module are:

RabbitMQReport - command ``/usr/sbin/rabbitmqctl report``
---------------------------------------------------------

RabbitMQReportOfContainers - files ``docker_exec_-t_rabbitmq-bundle-docker-*_rabbitmqctl_report``
-------------------------------------------------------------------------------------------------

RabbitMQUsers - command ``/usr/sbin/rabbitmqctl list_users``
------------------------------------------------------------

RabbitMQQueues - command ``/usr/sbin/rabbitmqctl list_queues name messages consumers auto_delete``
--------------------------------------------------------------------------------------------------

RabbitMQEnv - file ``/etc/rabbitmq/rabbitmq-env.conf``
------------------------------------------------------
"""
from collections import namedtuple
from re import compile

from insights.contrib import pyparsing as p
from insights.core import CommandParser, SysconfigOptions
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


# For "Status of node" section's erlang block prasing only, could not cover
# sections "Cluster status of node" & "Application environment of node".
def erlblock_parser():
    COMMA = p.Suppress(',')
    LBRACE, RBRACE = map(p.Suppress, "{}")
    LBRACKET, RBRACKET = map(p.Suppress, "[]")

    key = p.Word(p.alphas + '_')
    value_tnum = p.Word(p.nums + '.')
    value_tword = p.Word(p.alphanums + '/"-[]:.()\\')
    value_tstr = p.OneOrMore(value_tword)
    value_tdoustrs = value_tstr + COMMA + value_tstr
    value_tnumstr = value_tnum + value_tstr
    value = value_tdoustrs | value_tnumstr | value_tstr | value_tnum

    attr = LBRACE + p.Group(key + COMMA + value) + RBRACE
    attr_list = p.Dict(p.ZeroOrMore(attr + COMMA) + p.ZeroOrMore(attr))
    block = LBRACKET + attr_list + RBRACKET

    value_plus = block | attr | value
    attr = LBRACE + p.Group(key + COMMA + value_plus) + RBRACE
    attr_list = p.Dict(p.OneOrMore(attr + COMMA) + p.OneOrMore(attr))
    block = LBRACKET + attr_list + RBRACKET

    return block


# For "Permissions on" section parsing only
def perm_parser():
    COLON = p.Suppress(":")
    WHITE = p.Suppress(p.White())

    vhostname = p.Word(p.alphanums + '_-/')
    username = p.Word(p.alphanums + '_-')
    conf = p.Word(p.alphanums + '.*#')

    perm_vhost = p.Suppress("Permissions on") + vhostname + COLON
    ucwr = p.Suppress("user" + WHITE + "configure" + WHITE + "write" + WHITE + "read")
    perm_line = p.Group(username + 3 * (WHITE + conf))

    perm_con = p.Optional(ucwr + p.Dict(p.OneOrMore(perm_line)))
    perm_block = p.Group(perm_vhost + perm_con)
    perm = p.Dict(p.OneOrMore(perm_block))

    return perm


# Parsing "Status of node" & "Permissions on" sections, skip the other content.
def create_parser():
    DOTS = p.Suppress("...")
    NSTAT_PREFIX = p.Suppress("Status of node")
    PERM_PREFIX = p.Suppress("Permissions on")
    nodename = p.Word(p.alphanums + '\'_-@')

    block_nstat = p.Group(NSTAT_PREFIX + nodename + DOTS + erlblock_parser())
    nstat = p.Dict(p.OneOrMore(p.Suppress(p.SkipTo(NSTAT_PREFIX)) +
            block_nstat)).setResultsName('nstat')
    perm = p.Suppress(p.SkipTo(PERM_PREFIX)) + perm_parser().setResultsName('perm')

    return nstat + perm


@parser(Specs.rabbitmq_report)
class RabbitMQReport(CommandParser):

    def parse_content(self, content):
        """Support StatusOfNode and Permissions Sections only.

        Attrbutes:
            results(dict): None if encountered an error while parsing. For example::

                self.result =
                {'nstat': {
                    "'rabbit@overcloud-controller-0'": {
                        'file_descriptors': {
                            'total_used': '967',
                            'sockets_used': '965',
                            'total_limit': '3996',
                            'sockets_limit': '3594'},
                        'uptime': '3075485',
                        'pid': '6005',
                        'disk_free': '259739344896',
                        'disk_free_limit': '50000000'},
                    "'rabbit@overcloud-controller-1'": {
                        'file_descriptors': {
                            'total_used': '853',
                            'sockets_used': '851',
                            'total_limit': '3996',
                            'sockets_limit': '3594'},
                        'uptime': '3075482',
                        'pid': '9304',
                        'disk_free': '260561866752',
                        'disk_free_limit': '50000000'}}
                 'perm': {
                    '/': {
                        'redhat1': ['redhat.*', '.*', '.*'],
                        'guest': ['.*', '.*', '.*'],
                        'redhat':['redhat.*', '.*', '.*']},
                    'test_vhost': ''}}
        """
        # During the below parsing process, p.ParseException might be thrown.
        # No handler will be applied here.
        # And such p.ParseException won't be hidden, showing for debug usage.
        self.result = create_parser().parseString("\n".join(content)).asDict()


@parser(Specs.rabbitmq_report_of_containers)
class RabbitMQReportOfContainers(RabbitMQReport):
    """
    Parse the `rabbitmqctl report` command of each container running on the host.
    """
    pass


@parser(Specs.rabbitmq_users)
class RabbitMQUsers(CommandParser):

    def parse_content(self, content):
        self.data = {}
        for line in content[1:-1]:
            line_splits = line.split(None, 1)
            if len(line_splits) > 1:
                self.data[line_splits[0]] = line_splits[1][1:-1]


TRUE_FALSE = {'true': True, 'false': False}
"""dict: Dictionary for converting true/false strings to bool."""


@parser(Specs.rabbitmq_queues)
class RabbitMQQueues(CommandParser):
    """Parse the output of the `rabbitmqctl list_queues` command.

    The actual command is
    `rabbitmqctl list_queues name messages consumers auto_delete`.

    The four columns that are output are:

    1. name - The name of the queue with non-ASCII characters escaped as in C.
    2. messages - Sum of ready and unacknowledged messages (queue depth).
    3. consumers - Number of consumers.
    4. auto_delete - Whether the queue will be deleted automatically when no longer used.

    The output of the command looks like::

        cinder-scheduler        0       3       false
        cinder-scheduler.ha-controller  0       3       false
        cinder-scheduler_fanout_ea9c69fb630f41b2ae6120eba3cd43e0        8141    1   true
        cinder-scheduler_fanout_9aed9fbc3d4249289f2cb5ea04c062ab        8145    0   true
        cinder-scheduler_fanout_b7a2e488f3ed4e1587b959f9ac255b93        8141    0   true

    Examples:

        >>> queues.data[0]
        QueueInfo(name='cinder-scheduler', messages=0, consumers=3, auto_delete=False)
        >>> queues.data[0].name
        'cinder-scheduler'
        >>> queues.data[1].name
        'cinder-scheduler.ha-controller'

    Raises:
        ParseException: Raised if the data indicates an error in acquisition or if
            the auto_delete value is not true or false.
        ValueError: Raised if any of the numbers are not valid numbers
    """

    QueueInfo = namedtuple('QueueInfo', ['name', 'messages', 'consumers', 'auto_delete'])
    """namedtuple: Structure to hold a line of RabbitMQ queue information."""

    def parse_content(self, content):
        self.data = []
        for line in content:
            if "Listing queues ..." in line:
                continue
            if "...done." in line:
                continue
            parts = line.split()
            if len(parts) == 4 and not line.startswith('Error:'):
                if parts[3].lower() in TRUE_FALSE:
                    self.data.append(RabbitMQQueues.QueueInfo(
                        parts[0], int(parts[1]), int(parts[2]),
                        TRUE_FALSE[parts[3].lower()])
                    )
                else:
                    raise ParseException(
                        "auto_delete should be true or false: {0}".format(line))
            else:
                raise ParseException("Data appears invalid: {0}".format(line))


@parser(Specs.rabbitmq_env)
class RabbitMQEnv(SysconfigOptions):
    """Parse the content of file ``/etc/rabbitmq/rabbitmq-env.conf`` using
    the ``SysconfigOptions`` base class.


    Sample content of the file ``/etc/rabbitmq/rabbitmq-env.conf``::

        RABBITMQ_SERVER_ERL_ARGS="+K true +P 1048576 -kernel inet_default_connect_options [{nodelay,true},{raw,6,18,<<5000:64/native>>}] -kernel inet_default_listen_options [{raw,6,18,<<5000:64/native>>}]"

    Example:

        >>> rabbitmq_env.rabbitmq_server_erl_args
        '+K true +P 1048576 -kernel inet_default_connect_options [{nodelay,true},{raw,6,18,<<5000:64/native>>}] -kernel inet_default_listen_options [{raw,6,18,<<5000:64/native>>}]'
        >>> rabbitmq_env.data['RABBITMQ_SERVER_ERL_ARGS']
        '+K true +P 1048576 -kernel inet_default_connect_options [{nodelay,true},{raw,6,18,<<5000:64/native>>}] -kernel inet_default_listen_options [{raw,6,18,<<5000:64/native>>}]'
        >>> rabbitmq_env.rmq_erl_tcp_timeout
        '5000'

    Attributes:
        rabbitmq_server_erl_args(str): If RABBITMQ_SERVER_ERL_ARGS otherwise ``None``.

        rmq_erl_tcp_timeout(str): If value of inet_default_connect_options equals value of inet_default_listen_options. Otherwise ``None``.
    """
    @property
    def rabbitmq_server_erl_args(self):
        return self.data.get('RABBITMQ_SERVER_ERL_ARGS', None)

    @property
    def rmq_erl_tcp_timeout(self):
        pattern_connect_options = compile(r"inet_default_connect_options.*?\<<([0-9]*)\:64")
        pattern_listen_options = compile(r"inet_default_listen_options.*?\<<([0-9]*)\:64")

        if self.rabbitmq_server_erl_args:
            if pattern_connect_options.search(self.rabbitmq_server_erl_args) and pattern_listen_options.search(self.rabbitmq_server_erl_args):
                connect_timeout = pattern_connect_options.findall(self.rabbitmq_server_erl_args)[0]
                listen_timeout = pattern_listen_options.findall(self.rabbitmq_server_erl_args)[0]
                if connect_timeout == listen_timeout:
                    return connect_timeout



"""
Pacemaker configuration - file ``/var/lib/pacemaker/cib/cib.xml``
=================================================================

This parser reads the XML in the Pacemaker configuration file and provides a
standard ElementTree interface to it.  It also provides a ``nodes`` property
that lists all the nodes.

Sample input::

    <cib crm_feature_set="3.0.9" validate-with="pacemaker-2.3" have-quorum="1" dc-uuid="4">
      <configuration>
        <crm_config>
          <cluster_property_set id="cib-bootstrap-options">
            <nvpair id="cib-bootstrap-options-have-watchdog" name="have-watchdog" value="false"/>
            <nvpair id="cib-bootstrap-options-no-quorum-policy" name="no-quorum-policy" value="freeze"/>
          </cluster_property_set>
        </crm_config>
        <nodes>
          <node id="1" uname="foo"/>
          <node id="2" uname="bar"/>
          <node id="3" uname="baz"/>
        </nodes>
        <resources>
          <clone id="dlm-clone">
          </clone>
        </resources>
        <constraints>
          <rsc_order first="dlm-clone" first-action="start" id="order-dlm-clone-clvmd-clone-mandatory" then="clvmd-clone" then-action="start"/>
          <rsc_colocation id="colocation-clvmd-clone-dlm-clone-INFINITY" rsc="clvmd-clone" score="INFINITY" with-rsc="dlm-clone"/>
        </constraints>
      </configuration>
    </cib>

Examples:

    >>> cib = shared(CIB)
    >>> opts = cib.dom.find(".//cluster_property_set[@id='cib-bootstrap-options']")
    >>> opts.get('id')
    'cib-bootstrap-options'
    >>> cib.nodes
    ['foo', 'bar', 'baz']

"""

from .. import XMLParser, parser
from insights.specs import Specs


@parser(Specs.cib_xml)
class CIB(XMLParser):
    """
    Wraps a DOM of cib.xml

    self.dom is an instance of ElementTree.
    """

    @property
    def nodes(self):
        """
        Fetch the list of nodes and return their unames as a list.
        """
        return [n.get("uname").lower() for n in self.get_elements(".//nodes/node")]



"""
Mounts
======

This combiner provides information about mount entries.
More specifically this consolidates data from
:py:class:`insights.parsers.mount.Mount`,
:py:class:`insights.parsers.mount.ProcMounts` and
:py:class:`insights.parsers.mount.MountInfo`.

Examples:

    >>> type(mounts)
    <class 'insights.combiners.mounts.Mounts'>
    >>> len(mounts)
    19
    >>> '/boot' in mounts
    True
    >>> mounts['/boot'].mount_source
    '/dev/sda1'
    >>> mounts['/boot'].mount_options.get('data')
    'ordered'
    >>> mounts['/boot'].mount_addtlinfo.major_minor
    '8:1'
"""

import os
from insights.core.plugins import combiner
from insights.parsers import keyword_search
from insights.parsers.mount import Mount, ProcMounts, MountInfo
from insights.parsers.mount import MountEntry, MountAddtlInfo
from insights.util import deprecated


@combiner([Mount, ProcMounts, MountInfo])
class Mounts(object):
    """
    .. warning::
        This class is deprecated now and will be removed in version 3.2.0.
        Use insights.parsers.mount directly.

    ``Mounts`` combiner consolidates data from the parsers in
    ``insights.parsers.mounts`` module.

    Attributes:
        rows (list): list of :class:`MountEntry` objects for each mount entry
    """

    def __init__(self, binmount, procmounts, mountinfo):
        deprecated(Mounts, "Use the parsers in insights.parsers.mount module instead", "3.1.25")
        self._mounts = {}

        all_mount_points = set().union(
                    binmount.mounts.keys() if binmount else [],
                    procmounts.mounts.keys() if procmounts else [],
                    mountinfo.mounts.keys() if mountinfo else [])
        for mount_point in all_mount_points:
            this_binmount = binmount.get_dir(mount_point) if binmount else None
            this_procmounts = procmounts.get_dir(mount_point) if procmounts else None
            this_mountinfo = mountinfo.get_dir(mount_point) if mountinfo else None

            # create MountAddtlInfo
            addtlinfo = {}
            if this_binmount:
                if 'mount_label' in this_binmount:
                    addtlinfo['mount_label'] = this_binmount['mount_label']
                addtlinfo['mount_clause_binmount'] = this_binmount['mount_clause']
            if this_procmounts:
                addtlinfo['fs_freq'], addtlinfo['fs_passno'] = this_procmounts['mount_label']
                addtlinfo['mount_clause_procmounts'] = this_procmounts['mount_clause']
            if this_mountinfo:
                for k, v in this_mountinfo['mount_addtlinfo'].items():
                    addtlinfo[k] = v
                addtlinfo['mount_clause_mountinfo'] = this_mountinfo['mount_clause']
            entry_addtlinfo = MountAddtlInfo(addtlinfo)

            # create MountEntry
            source_mount = this_mountinfo if this_mountinfo else (
                            this_procmounts if this_procmounts else this_binmount)
            basicinfo = {}
            basicinfo['mount_point'] = mount_point
            basicinfo['mount_source'] = source_mount.get('mount_source') or source_mount.get('filesystem')
            basicinfo['mount_type'] = source_mount.get('mount_type')
            basicinfo['mount_options'] = source_mount.get('mount_options')
            basicinfo['mount_addtlinfo'] = entry_addtlinfo
            entry_info = MountEntry(basicinfo)

            self._mounts[mount_point] = entry_info
            self.rows = sorted(self._mounts.values(), key=lambda r:
                        (r.mount_addtlinfo.get('mount_id', '-1'), r.mount_point))

    @property
    def mount_points(self):
        """
        Returns:
            (list): list of `mount_point` for each mount entry
        """
        return list(self._mounts.keys())

    def get_dir(self, path):
        """
        This finds the most specific mount path that contains the given path,
        by successively removing the directory or file name on the end of
        the path and seeing if that is a mount point.  This will always
        terminate since / is always a mount point.  Strings that are not
        absolute paths will return None.

        Arguments:
            path (str): The path to check.
        Returns:
            MountEntry: The mount point that contains the given path.
        """
        while path != '':
            if path in self._mounts:
                return self._mounts[path]
            path = os.path.split(path)[0]
        return None

    def search(self, **kwargs):
        """
        Returns a list of the mounts matching the given criteria.
        Keys are searched for directly - see the
        :py:func:`insights.parsers.keyword_search` utility function for more
        details.  If no search parameters are given, no rows are returned.

        Arguments:
            **kwargs (dict): Dictionary of key-value pairs to search for.

        Returns:
            (list): The list of mount points matching the given criteria.

        Examples:

            >>> mounts.search(mount_point='/proc')[0]['mount_source']
            'proc'
            >>> mounts.search(mount_type__contains='nfs')[0]['mount_point']
            '/shared/dir1'
        """
        return keyword_search(self.rows, **kwargs)

    def __contains__(self, mount_point):
        """
        Check if a specific mount_point is mounted.

        Args:
            mount_point (str): a mount_point string

        Returns:
            bool: True if mount_point is mounted, otherwise False
        """
        return mount_point in self._mounts

    def __getitem__(self, mount_point):
        """
        Retrieve a specific mount entry by its mount_point.

        Args:
            mount_point (str): a mount_point string

        Returns:
            MountEntry : a object that represents a mount entry
        """
        return self._mounts.get(mount_point)

    def __len__(self):
        return len(self._mounts)

    def __iter__(self):
        for row in self.rows:
            yield row



import re


BLACKLISTED_SPECS = []
_FILE_FILTERS = set()
_COMMAND_FILTERS = set()
_PATTERN_FILTERS = set()
_KEYWORD_FILTERS = set()


def add_file(f):
    _FILE_FILTERS.add(re.compile(f))


def add_command(f):
    _COMMAND_FILTERS.add(re.compile(f))


def add_pattern(f):
    _PATTERN_FILTERS.add(f)


def add_keyword(f):
    _KEYWORD_FILTERS.add(f)


def allow_file(c):
    return not any(f.match(c) for f in _FILE_FILTERS)


def allow_command(c):
    return not any(f.match(c) for f in _COMMAND_FILTERS)


def get_disallowed_patterns():
    return _PATTERN_FILTERS


def get_disallowed_keywords():
    return _KEYWORD_FILTERS



import doctest
import pytest

from insights.parsers import spamassassin_channels
from insights.parsers.spamassassin_channels import SpamassassinChannels
from insights.tests import context_wrap

DEFAULT = """
/etc/mail/spamassassin/channel.d/sought.conf:CHANNELURL=sought.rules.yerp.org
/etc/mail/spamassassin/channel.d/spamassassin-official.conf:CHANNELURL=updates.spamassassin.org
""".strip()

EXPECTED_DEFAULT = {
    "/etc/mail/spamassassin/channel.d/sought.conf": ["sought.rules.yerp.org"],
    "/etc/mail/spamassassin/channel.d/spamassassin-official.conf": ["updates.spamassassin.org"],
}

SPACES = """
/etc/mail/spamassassin/channel.d/sought.conf:  CHANNELURL=sought.rules.yerp.org
/etc/mail/spamassassin/channel.d/spamassassin-official.conf:CHANNELURL=updates.spamassassin.org
""".strip()

EXPECTED_SPACES = {
    "/etc/mail/spamassassin/channel.d/sought.conf": ["sought.rules.yerp.org"],
    "/etc/mail/spamassassin/channel.d/spamassassin-official.conf": ["updates.spamassassin.org"],
}

TWO_IN_FILE = """
/etc/mail/spamassassin/channel.d/sought.conf:CHANNELURL=sought.rules.yerp.org
/etc/mail/spamassassin/channel.d/sought.conf:CHANNELURL=sought2.rules.yerp.org
/etc/mail/spamassassin/channel.d/spamassassin-official.conf:CHANNELURL=updates.spamassassin.org
""".strip()

EXPECTED_TWO_IN_FILE = {
    "/etc/mail/spamassassin/channel.d/sought.conf": ["sought.rules.yerp.org", "sought2.rules.yerp.org"],
    "/etc/mail/spamassassin/channel.d/spamassassin-official.conf": ["updates.spamassassin.org"],
}

INVALID = """
/etc/mail/spamassassin/channel.d/sought.conf:CHANNELURL=sought.rules.yerp.org
/etc/mail/spamassassin/channel.d/sought.conf:CHANNELURL=sought2.rules.yerp.org
garbage garbage
""".strip()


TEST_CASES = [
    (DEFAULT, EXPECTED_DEFAULT),
    (SPACES, EXPECTED_SPACES),
    (TWO_IN_FILE, EXPECTED_TWO_IN_FILE),
]


@pytest.mark.parametrize("input, output", TEST_CASES)
def test_spamassassin_channels(input, output):
    test = SpamassassinChannels(context_wrap(input))
    assert test.channels == output


def test_exception():
    with pytest.raises(Exception):
        SpamassassinChannels(context_wrap(INVALID))


def test_doc_examples():
    env = {
        "spamassassin_channels": SpamassassinChannels(context_wrap(DEFAULT)),
    }
    failed, total = doctest.testmod(spamassassin_channels, globs=env)
    assert failed == 0



from insights.parsers.modprobe import ModProbe
from insights.tests import context_wrap

MODPROBE_CONF = """
options ipv6 disable=1
options mlx4_core debug_level=1 log_num_mgm_entry_size=-1

install ipv6 /bin/true
""".strip()

MODPROBE_CONF_PATH = "etc/modprobe.conf"

MOD_OPTION_INFO = """
options ipv6 disable=1
options mlx4_core debug_level=1 log_num_mgm_entry_size=-1

install ipv6 /bin/true
""".strip()

MOD_OPTION_INFO_PATH = "etc/modprobe.d/ipv6.conf"

MOD_COMPLETE = """
#
# Syntax: see modprobe.conf(5).
#

# aliases
alias en* bnx2
alias eth* bnx2
alias scsi_hostadapter megaraid_sas
alias scsi_hostadapter1 ata_piix

# watchdog drivers
blacklist i8xx_tco

# Don't install the Firewire ethernet driver
install eth1394 /bin/true

# Special handling for USB mouse
install usbmouse /sbin/modprobe --first-time --ignore-install usbmouse && { /sbin/modprobe hid; /bin/true; }
remove usbmouse { /sbin/modprobe -r hid; } ; /sbin/modprobe -r --first-time --ignore-remove usbmouse

# bonding options
options bonding max_bonds=2
options bnx2 disable_msi=1

# Test bad data - save in bad_lines
alias
alias scsi_hostadapter2 ata_piix failed comment
balclkist ieee80211
""".strip()

MOD_COMPLETE_PATH = "etc/modprobe.conf"


def test_modprobe_v1():
    modprobe_info = ModProbe(context_wrap(MOD_OPTION_INFO,
                                          path=MOD_OPTION_INFO_PATH))
    assert modprobe_info["options"].get('ipv6') == ['disable=1']
    assert modprobe_info["options"].get('mlx4_core')[0] == 'debug_level=1'
    assert modprobe_info["install"].get('ipv6') == ['/bin/true']


def test_modprobe_v2():
    modprobe_info = ModProbe(context_wrap(MODPROBE_CONF,
                                          path=MODPROBE_CONF_PATH))
    assert modprobe_info["options"].get('ipv6') == ['disable=1']
    assert modprobe_info["options"].get('mlx4_core')[0] == 'debug_level=1'
    assert modprobe_info["install"].get('ipv6') == ['/bin/true']


def test_modprobe_complete():
    minfo = ModProbe(context_wrap(MOD_COMPLETE, path=MOD_COMPLETE_PATH))
    assert sorted(minfo.data.keys()) == sorted([
        'alias', 'blacklist', 'install', 'options', 'remove'
    ])
    assert sorted(minfo['alias']['bnx2']) == sorted(['eth*', 'en*'])
    assert 'i8xx_tco' in minfo['blacklist']
    assert minfo['blacklist']['i8xx_tco'] is True
    assert minfo['install']['eth1394'] == ['/bin/true']
    assert 'usbmouse' in minfo['install']
    assert 'usbmouse' in minfo['remove']
    assert minfo['options']['bonding'] == ['max_bonds=2']
    assert minfo['options']['bnx2'] == ['disable_msi=1']

    # Test that we get bad lines
    assert len(minfo.bad_lines) == 3
    assert minfo.bad_lines[0] == 'alias'
    assert minfo.bad_lines[1] == 'alias scsi_hostadapter2 ata_piix failed comment'
    assert minfo.bad_lines[2] == 'balclkist ieee80211'



"""
CmdLine - file ``/proc/cmdline``
================================

This parser reads the ``/proc/cmdline`` file, which contains the arguments
given to the currently running kernel on boot.

"""
from insights.core import LegacyItemAccess, Parser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.cmdline)
class CmdLine(LegacyItemAccess, Parser):
    """
    A parser class for parsing the Linux kernel command line as given in
    ``/proc/cmdline``.

    Parsing Logic::

        Parses all elements in command line to a dict where the key is the
        element itself and the value is a list stores its corresponding values.

        If an element doesn't contain "=", set the corresponding value to `True`.

        If an element contains "=", set the corresponding value to the whole right
        value of the "=".

    .. note::

        For special command line elements that include two "=", e.g.
        ``root=LABEL=/1``, "root" will be the key and "LABEL=/1" will be
        the value in the returned list.

        Some parameters (the returned keys) might be still effective
        even if there is '#' before it, e.g.: ``#rhgb``.  This should
        be checked by the rule.

    Sample input::

        BOOT_IMAGE=/vmlinuz-3.10.0-327.36.3.el7.x86_64 root=/dev/system_vg/Root ro rd.lvm.lv=system_vg/Root crashkernel=auto rd.lvm.lv=system_vg/Swap rhgb quiet LANG=en_GB.utf8

    Examples:
        >>> cmd['BOOT_IMAGE']
        ['/vmlinuz-3.10.0-327.36.3.el7.x86_64']
        >>> cmd['rd.lvm.lv']
        ['system_vg/Root', 'system_vg/Swap']
        >>> 'autofs' in cmd
        False
        >>> cmd.get('autofs') is None
        True
        >>> 'quiet' in cmd
        True
        >>> cmd.get('quiet')
        [True]
        >>> cmd['crashkernel']
        ['auto']

    Attributes:
        data (dict): Parsed booting arguments are stored in this dictionary
        cmdline (str): The RAW line of the ``/proc/cmdline``
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent('Empty output')
        if len(content) != 1:
            raise ParseException('Invalid output: {0}', content)

        self.data = {}
        self.cmdline = content[0].strip()
        for el in self.cmdline.split():
            # Thanks to strip and split, el always contains something.
            key, value = el, True
            if "=" in el:
                key, value = el.split("=", 1)
            if key not in self.data:
                self.data[key] = []
            self.data[key].append(value)



"""
Custom datasources for ps information
"""
import json
import os.path

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_command
from insights.specs import Specs


class LocalSpecs(Specs):
    """ Local specs used only by ps datasources """

    ps_eo_args = simple_command("/bin/ps -ewwo pid,args")
    """ Returns ps output including pid and full args """


@datasource(LocalSpecs.ps_eo_args, HostContext)
def ps_eo_cmd(broker):
    """
    Custom datasource to collect the full paths to all running commands on the system
    provided by the ``ps -ewwo pid,args`` command.  After collecting the data, all of the
    args are trimmed to leave only the command including full path.

    Sample output from the ``ps -ewwo pid, args`` command::

        PID COMMAND
          1 /usr/lib/systemd/systemd --switched-root --system --deserialize 31
          2 [kthreadd]
          3 [rcu_gp]
          4 [rcu_par_gp]
          6 [kworker/0:0H-events_highpri]
          9 [mm_percpu_wq]
         10 [rcu_tasks_kthre]
         11 /usr/bin/python3 /home/user1/python_app.py
         12 [kworker/u16:0-kcryptd/253:0]

    This datasource trims off the args to minimize possible PII and sensitive information.
    After trimming the data looks like this::

        PID COMMAND
          1 /usr/lib/systemd/systemd
          2 [kthreadd]
          3 [rcu_gp]
          4 [rcu_par_gp]
          6 [kworker/0:0H-events_highpri]
          9 [mm_percpu_wq]
         10 [rcu_tasks_kthre]
         11 /usr/bin/python3
         12 [kworker/u16:0-kcryptd/253:0]

    Returns:
        str: Returns a multiline string in the same format as ``ps`` output

    Raises:
        SkipComponent: Raised if no data is available
    """
    content = broker[LocalSpecs.ps_eo_args].content
    data = []
    data.append('PID COMMAND')
    start = False
    for l in content:
        if 'PID' in l and 'COMMAND' in l:
            start = True
            continue
        if not start:
            continue
        pid, args = l.strip().split(None, 1)
        if ' ' in args:
            cmd, _ = args.split(None, 1)
        else:
            cmd = args
        data.append('{0} {1}'.format(pid, cmd))

    if len(data) > 1:
        return DatasourceProvider('\n'.join(data), relative_path='insights_commands/ps_eo_cmd')

    raise SkipComponent()


@datasource(LocalSpecs.ps_eo_args, HostContext)
def jboss_runtime_versions(broker):
    """
     Custom datasource to collect the <JBOSS_HOME>/version.txt.

     Sample output from the ``ps -ewwo pid, args`` command::

         PID COMMAND
           1 /usr/lib/systemd/systemd --switched-root --system --deserialize 31
           2 [kthreadd]
           3 [rcu_gp]
           4 [rcu_par_gp]
           6 [kworker/0:0H-events_highpri]
           8686 java -D[Standalone] -server -verbose:gc -Xms64m -Xmx512m -Djboss.home.dir=/opt/jboss-datagrid-7.3.0-server -Djboss.server.base.dir=/opt/jboss-datagrid-7.3.0-server/standalone


     Get the Jboss home directory and read the version.txt::

         -Djboss.home.dir=/opt/jboss-datagrid-7.3.0-server
         /opt/jboss-datagrid-7.3.0-server/version.txt

     Returns:
         str: string of dict {<jboss_home>: <content of version.txt>}

     Raises:
         SkipComponent: Raised if no data is available
     """
    content = broker[LocalSpecs.ps_eo_args].content
    jboss_home_dirs = set()
    data = {}
    for l in content:
        if 'java ' in l:
            jboss_home_labels = ['-jboss-home ', '-Djboss.home.dir=', '-Dcatalina.home=',
                                 '-Dinfinispan.server.home.path=']
            for jhl in jboss_home_labels:
                if jhl in l:
                    jboss_home_str = l.split(jhl)[1]
                    if jboss_home_str.startswith('/'):
                        jboss_home_dirs.add(jboss_home_str.split()[0])
    if jboss_home_dirs:
        for one_jboss_home_dir in jboss_home_dirs:
            jboss_v_file = os.path.join(one_jboss_home_dir, 'version.txt')
            if os.path.exists(jboss_v_file):
                with open(jboss_v_file, 'r') as version_file:
                    data[one_jboss_home_dir] = version_file.read()
    if len(data) > 0:
        return DatasourceProvider(json.dumps(data), relative_path='insights_commands/jboss_versions')
    raise SkipComponent()



import doctest
import pytest

from insights.core.exceptions import ParseException
from insights.parsers import df
from insights.tests import context_wrap

DF_LI = """
Filesystem        Inodes  IUsed     IFree IUse% Mounted on
/dev/mapper/vg_lxcrhel6sat56-lv_root

                 6275072 124955   6150117    2% /

devtmpfs         1497120    532   1496588    1% /dev
tmpfs            1499684    331   1499353    1% /dev/shm
tmpfs            1499684    728   1498956    1% /run
tmpfs            1499684     16   1499668    1% /sys/fs/cgroup
tmpfs            1499684     54   1499630    1% /tmp
/dev/sda2      106954752 298662 106656090    1% /home
/dev/sda1         128016    429    127587    1% /boot
/dev/sdb1        1499684      6   1499678    1% /V M T o o l s
/dev/sdb2        1499684     15   1499669    1% /VM Tools
""".strip()


def test_df_li():
    df_list = df.DiskFree_LI(context_wrap(DF_LI))
    assert len(df_list) == 10
    assert len(df_list.mounts) == 10
    assert len(df_list.filesystems) == 7
    assert '/home' in df_list.mounts
    r = df.Record(
        filesystem='/dev/sda2',
        total='106954752',
        used='298662',
        available='106656090',
        capacity='1%',
        mounted_on='/home'
    )

    assert df_list.get_mount('/home') == r
    assert '/dev/sda2' in df_list.filesystems
    assert len(df_list.get_filesystem('/dev/sda2')) == 1
    assert df_list.get_filesystem('/dev/sda2')[0] == r
    assert len(df_list.get_filesystem('tmpfs')) == 4
    assert df_list.get_mount('/dev').filesystem == 'devtmpfs'
    assert df_list.get_mount('/run').total == '1499684'
    assert df_list.get_mount('/tmp').used == '54'
    assert df_list.get_mount('/boot').available == '127587'
    assert df_list.get_filesystem('/dev/sda2')[0].capacity == '1%'
    assert df_list.get_filesystem('/dev/sda2')[0].available == '106656090'
    assert df_list.get_filesystem('devtmpfs')[0].mounted_on == '/dev'
    assert df_list.get_mount('/V M T o o l s').available == '1499678'
    assert df_list.get_filesystem('/dev/mapper/vg_lxcrhel6sat56-lv_root')[0].mounted_on == '/'
    sorted_mount_names = sorted([
        '/', '/dev', '/dev/shm', '/run', '/sys/fs/cgroup', '/tmp', '/home',
        '/boot', '/V M T o o l s', '/VM Tools'
    ])
    assert sorted([d.mounted_on for d in df_list]) == sorted_mount_names
    assert sorted(df_list.mount_names) == sorted_mount_names
    assert sorted(df_list.filesystem_names) == sorted([
        '/dev/mapper/vg_lxcrhel6sat56-lv_root', 'devtmpfs', 'tmpfs',
        '/dev/sda2', '/dev/sda1', '/dev/sdb2', '/dev/sdb1'
    ])

    # Test get_path
    # Root mount point works:
    assert df_list.get_dir('/') == df_list.get_mount('/')
    # Mount point underneath root works:
    assert df_list.get_dir('/dev') == df_list.get_mount('/dev')
    # Directory underneath sub-mount:
    assert df_list.get_dir('/boot/grub2') == df_list.get_mount('/boot')
    # Directory with / suffix:
    assert df_list.get_dir('/boot/grub2/') == df_list.get_mount('/boot')
    # File path also works:
    assert df_list.get_dir('/boot/grub2/grub.cfg') == df_list.get_mount('/boot')
    # Relative path returns None
    assert df_list.get_dir('var/lib') is None
    # Invalid path returns None
    assert df_list.get_dir('"') is None


DF_ALP = """
/bin/df: '/vobs/GEMS': No such file or directory
/bin/df: '/vobs/NT/TFax': No such file or directory
Filesystem                           1024-blocks      Used Available Capacity Mounted on
/dev/mapper/vg_lxcrhel6sat56-lv_root    98571884   4244032  89313940       5% /
sysfs                                          0         0         0        - /sys
proc                                           0         0         0        - /proc
devtmpfs                                 5988480         0   5988480       0% /dev
securityfs                                     0         0         0        - /sys/kernel/security
tmpfs                                    5998736    491660   5507076       9% /dev/shm
devpts                                         0         0         0        - /dev/pts
tmpfs                                    5998736      1380   5997356       1% /run

tmpfs                                    5998736         0   5998736       0% /sys/fs/cgroup
""".strip()


def test_df_alP():
    df_list = df.DiskFree_ALP(context_wrap(DF_ALP))
    assert len(df_list) == 9
    assert len(df_list.mounts) == 9
    assert len(df_list.filesystems) == 7
    assert '/' in df_list.mounts
    r = df.Record(
        filesystem='/dev/mapper/vg_lxcrhel6sat56-lv_root',
        total='98571884',
        used='4244032',
        available='89313940',
        capacity='5%',
        mounted_on='/'
    )
    assert df_list.get_mount('/') == r
    assert '/dev/mapper/vg_lxcrhel6sat56-lv_root' in df_list.filesystems
    assert len(df_list.get_filesystem('/dev/mapper/vg_lxcrhel6sat56-lv_root')) == 1
    assert df_list.get_filesystem('/dev/mapper/vg_lxcrhel6sat56-lv_root')[0] == r
    assert len(df_list.get_filesystem('tmpfs')) == 3
    assert df_list.get_mount('/sys').filesystem == 'sysfs'
    assert df_list.get_mount('/proc').total == '0'
    assert df_list.get_mount('/dev').used == '0'
    assert df_list.get_mount('/run').available == '5997356'
    assert df_list.get_mount('/sys/fs/cgroup').capacity == '0%'
    assert df_list.get_mount('/').filesystem == '/dev/mapper/vg_lxcrhel6sat56-lv_root'
    assert df_list.get_mount('/').capacity == '5%'

    # Test get_path
    # Root mount point works:
    assert df_list.get_dir('/') == df_list.get_mount('/')
    # Mount point underneath root works:
    assert df_list.get_dir('/dev') == df_list.get_mount('/dev')
    # Directory underneath sub-mount:
    assert df_list.get_dir('/dev/v4l') == df_list.get_mount('/dev')
    # Directory with / suffix:
    assert df_list.get_dir('/dev/v4l/') == df_list.get_mount('/dev')
    # File path also works:
    assert df_list.get_dir('/dev/v4l/adapter0/control0.cfg') == df_list.get_mount('/dev')
    # Relative path returns None
    assert df_list.get_dir('dev/sys') is None
    # Invalid path returns None
    assert df_list.get_dir('"') is None


DF_AL = """
Filesystem                             1K-blocks      Used Available     Use% Mounted on
/dev/mapper/vg_lxcrhel6sat56-lv_root    98571884   4244032  89313940       5% /
sysfs                                          0         0         0        - /sys
proc                                           0         0         0        - /proc
devtmpfs                                 5988480         0   5988480       0% /dev
securityfs                                     0         0         0        - /sys/kernel/security

tmpfs                                    5998736    491660   5507076       9% /dev/shm
devpts                                         0         0         0        - /dev/pts
tmpfs                                    5998736      1380   5997356       1% /run
tmpfs                                    5998736         0   5998736       0% /sys/fs/cgroup
""".strip()


def test_df_al():
    df_list = df.DiskFree_AL(context_wrap(DF_AL))
    assert len(df_list) == 9
    assert len(df_list.mounts) == 9
    assert len(df_list.filesystems) == 7
    assert '/' in df_list.mounts
    r = df.Record(
        filesystem='/dev/mapper/vg_lxcrhel6sat56-lv_root',
        total='98571884',
        used='4244032',
        available='89313940',
        capacity='5%',
        mounted_on='/'
    )
    assert df_list.get_mount('/') == r
    assert '/dev/mapper/vg_lxcrhel6sat56-lv_root' in df_list.filesystems
    assert len(df_list.get_filesystem('/dev/mapper/vg_lxcrhel6sat56-lv_root')) == 1
    assert df_list.get_filesystem('/dev/mapper/vg_lxcrhel6sat56-lv_root')[0] == r
    assert len(df_list.get_filesystem('tmpfs')) == 3
    assert df_list.get_mount('/sys').filesystem == 'sysfs'
    assert df_list.get_mount('/proc').total == '0'
    assert df_list.get_mount('/dev').used == '0'
    assert df_list.get_mount('/run').available == '5997356'
    assert df_list.get_mount('/sys/fs/cgroup').capacity == '0%'
    assert df_list.get_mount('/').filesystem == '/dev/mapper/vg_lxcrhel6sat56-lv_root'
    assert df_list.get_mount('/').capacity == '5%'

    # Test get_path
    # Root mount point works:
    assert df_list.get_dir('/') == df_list.get_mount('/')
    # Mount point underneath root works:
    assert df_list.get_dir('/dev') == df_list.get_mount('/dev')
    # Directory underneath sub-mount:
    assert df_list.get_dir('/dev/v4l') == df_list.get_mount('/dev')
    # Directory with / suffix:
    assert df_list.get_dir('/dev/v4l/') == df_list.get_mount('/dev')
    # File path also works:
    assert df_list.get_dir('/dev/v4l/adapter0/control0.cfg') == df_list.get_mount('/dev')
    # Relative path returns None
    assert df_list.get_dir('dev/sys') is None
    # Invalid path returns None
    assert df_list.get_dir('"') is None


DF_AL_BAD = """
Filesystem                             1K-blocks      Used Available     Use% Mounted on
/dev/mapper/vg_lxcrhel6sat56-lv_root    98571884   4244032  89313940       5% /
sysfs                                          0
"""

DF_AL_BAD_BS = """
Filesystem                             1a-blocks      Used Available     Use% Mounted on
/dev/mapper/vg_lxcrhel6sat56-lv_root    98571884   4244032  89313940       5% /
"""


def test_df_al_bad():
    with pytest.raises(ParseException) as exc:
        df_list = df.DiskFree_AL(context_wrap(DF_AL_BAD))
        assert len(df_list) == 2
    assert 'Could not parse line' in str(exc)

    with pytest.raises(ParseException) as exc:
        df.DiskFree_AL(context_wrap(DF_AL_BAD_BS))
    assert 'Unknown block size' in str(exc)


DF_AL_BS_2MB = """
Filesystem     2MB-blocks  Used Available Use% Mounted on
/dev/vda3           62031 49197      9680  84% /
"""


def test_df_al_2MB():
    df_list = df.DiskFree_LI(context_wrap(DF_AL_BS_2MB))
    root = df_list.get_mount('/')
    assert root.filesystem == '/dev/vda3'
    assert root.total == '62031'
    assert df_list.raw_block_size == '2MB'
    assert df_list.block_size == 2000000
    assert int(root.total) * df_list.block_size == 124062000000  # To Bytes


DF_LI_DOC = """
Filesystem       Inodes IUsed    IFree IUse% Mounted on
devtmpfs         242224   359   241865    1% /dev
tmpfs            246028     1   246027    1% /dev/shm
tmpfs            246028   491   245537    1% /run
tmpfs            246028    17   246011    1% /sys/fs/cgroup
/dev/sda2       8911872 58130  8853742    1% /
/dev/sdb1      26213888 19662 26194226    1% /opt/data
/dev/sda1        524288   306   523982    1% /boot
tmpfs            246028     5   246023    1% /run/user/0
""".strip()

DF_ALP_DOC = """
Filesystem     1024-blocks    Used Available Capacity Mounted on
sysfs                    0       0         0        - /sys
proc                     0       0         0        - /proc
devtmpfs            968896       0    968896       0% /dev
securityfs               0       0         0        - /sys/kernel/security
tmpfs               984112       0    984112       0% /dev/shm
devpts                   0       0         0        - /dev/pts
tmpfs               984112    8660    975452       1% /run
tmpfs               984112       0    984112       0% /sys/fs/cgroup
cgroup                   0       0         0        - /sys/fs/cgroup/systemd
cgroup                   0       0         0        - /sys/fs/cgroup/pids
cgroup                   0       0         0        - /sys/fs/cgroup/rdma
configfs                 0       0         0        - /sys/kernel/config
/dev/sda2         17813504 2127172  15686332      12% /
selinuxfs                0       0         0        - /sys/fs/selinux
systemd-1                -       -         -        - /proc/sys/fs/binfmt_misc
debugfs                  0       0         0        - /sys/kernel/debug
mqueue                   0       0         0        - /dev/mqueue
hugetlbfs                0       0         0        - /dev/hugepages
/dev/sdb1         52402180 1088148  51314032       3% /V M T o o l s
/dev/sda1          1038336  185676    852660      18% /boot
""".strip()

DF_AL_DOC = """
Filesystem     1K-blocks    Used Available Use% Mounted on
sysfs                  0       0         0    - /sys
proc                   0       0         0    - /proc
devtmpfs          968896       0    968896   0% /dev
securityfs             0       0         0    - /sys/kernel/security
tmpfs             984112       0    984112   0% /dev/shm
devpts                 0       0         0    - /dev/pts
tmpfs             984112    8660    975452   1% /run
tmpfs             984112       0    984112   0% /sys/fs/cgroup
cgroup                 0       0         0    - /sys/fs/cgroup/systemd
cgroup                 0       0         0    - /sys/fs/cgroup/pids
cgroup                 0       0         0    - /sys/fs/cgroup/rdma
configfs               0       0         0    - /sys/kernel/config
/dev/sda2       17813504 2127172  15686332  12% /
selinuxfs              0       0         0    - /sys/fs/selinux
systemd-1              -       -         -    - /proc/sys/fs/binfmt_misc
debugfs                0       0         0    - /sys/kernel/debug
mqueue                 0       0         0    - /dev/mqueue
hugetlbfs              0       0         0    - /dev/hugepages
/dev/sdb1       52402180 1088148  51314032   3% /V M T o o l s
/dev/sda1        1038336  185676    852660  18% /boot
""".strip()


def test_doc_examples():
    env = {
            'df_li': df.DiskFree_LI(context_wrap(DF_LI_DOC)),
            'df_al': df.DiskFree_AL(context_wrap(DF_AL_DOC)),
            'df_alP': df.DiskFree_ALP(context_wrap(DF_ALP_DOC)),
          }
    failed, total = doctest.testmod(df, globs=env)
    assert failed == 0



"""
Dump the platform nvdimm device topology and attributes in json
===============================================================

This module contains the following parsers:

NdctlListNi - command ``/usr/bin/ndctl list -Ni``
=================================================
"""

from insights.core import JSONParser, CommandParser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.ndctl_list_Ni)
class NdctlListNi(JSONParser, CommandParser):
    """
    Class for parsing the command of ``/usr/bin/ndctl list -Ni``

    Sample output::

        [
            {
                "dev":"namespace1.0",
                "mode":"fsdax",
                "map":"mem",
                "size":811746721792,
                "uuid":"6a5d93a5-6044-461b-8d19-0409bd323a94",
                "sector_size":512,
                "align":2097152,
                "blockdev":"pmem1"
            },
            {
                "dev":"namespace1.1",
                "mode":"raw",
                "size":0,
                "uuid":"00000000-0000-0000-0000-000000000000",
                "sector_size":512,
                "state":"disabled"
            },
            {
                "dev":"namespace0.0",
                "mode":"raw",
                "size":0,
                "uuid":"00000000-0000-0000-0000-000000000000",
                "sector_size":512,
                "state":"disabled"
            }
        ]

    Examples:

        >>> type(ndctl_list)
        <class 'insights.parsers.ndctl_list.NdctlListNi'>
        >>> 'pmem1' in ndctl_list.blockdev_list
        True
        >>> ndctl_list.get_blockdev('pmem1').get('mode') == 'fsdax'
        True
    """

    def parse_content(self, content):
        super(NdctlListNi, self).parse_content(content)
        self._blockdevs = [item['blockdev'] for item in self.data if 'blockdev' in item]

    @property
    def blockdev_list(self):
        """ Return a list of the blockdev attribute of all the devices if it has the attribute"""
        return self._blockdevs

    def get_blockdev(self, dev_name):
        """
        Return a dict of the block device info

        Args:
            dev_name (str): the blockdev name

        Returns:
            dict: return a dict with all the info if there is the block device else empty dict

        """
        for item in self.data:
            if item.get('blockdev', '') == dev_name:
                return item
        return {}



import pytest

from mock.mock import patch, PropertyMock

from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.spec_factory import ContainerCommandProvider
from insights.specs.datasources.container.nginx_conf import LocalSpecs, nginx_conf


find_list = [
    '/etc/nginx/nginx.conf',
    '/etc/nginx/nginx.conf.d/ss-nginx.conf',
    '/opt/rh-nginx/nginx.conf',
    '/opt/rh-nginx/nginx.conf.d/ss-nginx.conf',
]

find_list_ng = [
    '/etc/rd-nginx/nginx.conf',
    '/etc/rd-nginx/nginx.conf.d/ss-nginx.conf',
    '/opt/nginx/nginx.conf',
    '/opt/nginx/nginx.conf.d/ss-nginx.conf',
]


@patch("insights.core.spec_factory.CommandOutputProvider.validate", return_value=None)
@patch("insights.core.spec_factory.CommandOutputProvider.load", return_value=None)
@patch("insights.core.spec_factory.CommandOutputProvider.content",
        new_callable=PropertyMock, return_value=find_list)
def test_nginx_conf(validate, load, find_l):
    find_nginx_confs = [
        ContainerCommandProvider(
            'podman exec 03e2861336a7 find /etc/ /opt/ *.conf',
            None, 'rhel8_nginx'),
        ContainerCommandProvider(
            'docker exec 03e2861336a8 find /etc/ /opt/ *.conf',
            None, 'rhel7_nginx'),
    ]
    broker = {
        LocalSpecs.container_find_etc_opt_conf: find_nginx_confs,
        HostContext: None}

    ret = nginx_conf(broker)
    assert len(ret) == 8
    assert ('rhel8_nginx', 'podman', '03e2861336a7', '/etc/nginx/nginx.conf') in ret
    assert ('rhel7_nginx', 'docker', '03e2861336a8', '/opt/rh-nginx/nginx.conf.d/ss-nginx.conf') in ret


@patch("insights.core.spec_factory.CommandOutputProvider.validate", return_value=None)
@patch("insights.core.spec_factory.CommandOutputProvider.load", return_value=None)
@patch("insights.core.spec_factory.CommandOutputProvider.content",
        new_callable=PropertyMock, return_value=find_list_ng)
def test_nginx_conf_empty(validate, load, find_l):
    find_nginx_confs_ng = [
        ContainerCommandProvider(
            'podman exec 03e2861336a7 find /etc/ /opt/ *.conf',
            None, 'rhel8_nginx'),
        ContainerCommandProvider(
            'docker exec 03e2861336a8 find /etc/ /opt/ *.conf',
            None, 'rhel7_nginx'),
    ]
    broker = {
        LocalSpecs.container_find_etc_opt_conf: find_nginx_confs_ng,
        HostContext: None}

    with pytest.raises(SkipComponent):
        ret = nginx_conf(broker)
        assert len(ret) == 0



"""
SatelliteEnabledFeatures - command ``curl -sk https://localhost:9090/features --connect-timeout 5``
===================================================================================================

The satellite enabled features parser reads the output of
``curl -sk https://localhost:9090/features --connect-timeout 5`` and convert it into a list.

Sample output of ``curl -sk https://localhost:9090/features --connect-timeout 5``::

    ["ansible","dhcp","discovery",dynflow","logs","openscap","pulp","puppet","puppetca","ssh","templates","tftp"]

Examples:

    >>> type(satellite_features)
    <class 'insights.parsers.satellite_enabled_features.SatelliteEnabledFeatures'>
    >>> 'dhcp' in satellite_features
    True
    >>> 'dns' in satellite_features
    False
"""
from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.satellite_enabled_features)
class SatelliteEnabledFeatures(CommandParser, list):
    """
    Read the ``curl -sk https://localhost:9090/features --connect-timeout 5`` command and
    convert it to a list.
    """

    def parse_content(self, content):
        for line in content:
            features = line.strip('[]')
            self.extend([feature.strip('"') for feature in features.split(',') if feature])
        if len(self) == 0:
            raise SkipComponent



"""
IPA - Combiner for RHEL IdM / FreeIPA information
=================================================
"""
from insights.core.plugins import combiner
from insights.core.exceptions import SkipComponent
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.redhat_release import RedhatRelease
from insights.parsers.ipa_conf import IPAConfig
from insights.parsers.sssd_conf import SSSD_Config


@combiner(IPAConfig, SSSD_Config, InstalledRpms, RedhatRelease)
class IPA(object):
    """Combiner for IPA, SSSD, and installed RPMs

    Provides additional information, e.g. whether the host is an IPA server.
    """

    def __init__(self, ipa_conf, sssd_conf, rpms, release):
        self._ipa_conf = ipa_conf
        self._sssd_conf = sssd_conf
        # IPA package names are different on Fedora
        if release.is_fedora:
            self._client_rpm = rpms.get_max("freeipa-client")
            self._server_rpm = rpms.get_max("freeipa-server")
        else:
            self._client_rpm = rpms.get_max("ipa-client")
            self._server_rpm = rpms.get_max("ipa-server")
        if self._client_rpm is None:
            raise SkipComponent("IPA client package is not installed")
        self._is_client = None
        self._is_server = None

    @property
    def ipa_conf(self):
        """Get IPAConfig object"""
        return self._ipa_conf

    @property
    def sssd_conf(self):
        """Get SSSD_Config object"""
        return self._sssd_conf

    @property
    def sssd_domain_config(self):
        """Get SSSD domain configuration for host's IPA domain"""
        return self._sssd_conf.domain_config(self._ipa_conf.domain)

    @property
    def is_client(self):
        """Is the host an IPA client?"""
        # IPAConfig validates that /etc/ipa/default.conf exists and is a
        # valid IPA config file with all required values present.
        if self._is_client is None:
            id_provider = self.sssd_domain_config.get("id_provider")
            if id_provider == "ipa":
                self._is_client = True
            else:
                self._is_client = False

        return self._is_client

    @property
    def is_server(self):
        """Is the host an IPA server?"""
        if self._is_server is None:
            server_mode = self.sssd_domain_config.get(
                "ipa_server_mode", "false"
            )
            if (
                self._server_rpm and
                # all servers are also clients
                self.is_client and
                # only servers use LDAPI (LDAP over Unix socket)
                self._ipa_conf.ldap_uri.startswith("ldapi://") and
                # SSSD domain must be in server mode
                server_mode.lower() == "true"
            ):
                self._is_server = True
            else:
                self._is_server = False

        return self._is_server



"""
LsVarLibRpm - command ``ls -lan /var/lib/rpm``
==============================================
"""


from insights.core import CommandParser, FileListing
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.ls_var_lib_rpm)
class LsVarLibRpm(CommandParser, FileListing):
    """
    Parses output of ``ls -lan /var/lib/rpm`` command.

    Sample output::

        total 172400
        drwxr-xr-x.  2 0 0      4096 Oct 19  2022 .
        drwxr-xr-x. 31 0 0      4096 Nov 26  2021 ..
        -rw-r--r--.  1 0 0   4034560 Apr 17 16:06 Basenames
        -rw-r--r--.  1 0 0      8192 Apr 17 16:06 Conflictname
        -rw-r--r--.  1 0 0     16384 Apr 17 10:43 Obsoletename
        -rw-r--r--.  1 0 0 165253120 Apr 17 16:06 Packages
        -rw-r--r--.  1 0 0   2461696 Apr 17 16:06 Providename
        -rw-r--r--.  1 0 0      8192 Apr 17 10:42 Recommendname

    Examples:
        >>> type(var_lib_rpm)
        <class 'insights.parsers.ls_var_lib_rpm.LsVarLibRpm'>
        >>> var_lib_rpm.dir_contains('/var/lib/rpm', 'Packages')
        True
        >>> var_lib_rpm.files_of('/var/lib/rpm')
        ['Basenames', 'Conflictname', 'Obsoletename', 'Packages', 'Providename', 'Recommendname']

    """
    pass



import doctest

from insights.parsers import ls_var_cache_pulp
from insights.parsers.ls_var_cache_pulp import LsVarCachePulp
from insights.tests import context_wrap

LS_VAR_CACHE_PULP = """
total 0
drwxrwxr-x.  5 48 1000 216 Jan 21 12:56 .
drwxr-xr-x. 10  0    0 121 Jan 20 13:57 ..
lrwxrwxrwx.  1  0    0  19 Jan 21 12:56 cache -> /var/lib/pulp/cache
drwxr-xr-x.  2 48   48   6 Jan 21 13:03 reserved_resource_worker-0@dhcp130-202.gsslab.pnq2.redhat.com
drwxr-xr-x.  2 48   48   6 Jan 21 02:03 reserved_resource_worker-1@dhcp130-202.gsslab.pnq2.redhat.com
drwxr-xr-x.  2 48   48   6 Jan 20 14:03 resource_manager@dhcp130-202.gsslab.pnq2.redhat.com
"""


def test_ls_var_cache_pulp():
    ls_var_cache_pulp = LsVarCachePulp(context_wrap(LS_VAR_CACHE_PULP, path="insights_commands/ls_-lan_.var.cache.pulp"))
    assert ls_var_cache_pulp.files_of('/var/cache/pulp') == ['cache']
    cache_item = ls_var_cache_pulp.dir_entry('/var/cache/pulp', 'cache')
    assert cache_item is not None
    assert '/var/lib/pulp/' in cache_item['link']


def test_ls_var_lib_mongodb_doc_examples():
    env = {
        'ls_var_cache_pulp': LsVarCachePulp(context_wrap(LS_VAR_CACHE_PULP, path="insights_commands/ls_-lan_.var.cache.pulp")),
    }
    failed, total = doctest.testmod(ls_var_cache_pulp, globs=env)
    assert failed == 0



import doctest

from insights.parsers import heat_conf
from insights.tests import context_wrap

HEAT_CONFIG = """
[DEFAULT]
heat_metadata_server_url = http://172.16.0.11:8000
heat_waitcondition_server_url = http://172.16.0.11:8000/v1/waitcondition
heat_watch_server_url =http://172.16.0.11:8003
stack_user_domain_name = heat_stack
stack_domain_admin = heat_stack_domain_admin
stack_domain_admin_password = *********
auth_encryption_key = mysupersecretkey
log_dir = /var/log/heat
instance_user=
notification_driver=messaging
[auth_password]
[clients]
[clients_ceilometer]
[clients_cinder]
[clients_glance]
[clients_heat]
[clients_keystone]
auth_uri =http://192.0.2.18:35357
[clients_neutron]
[clients_nova]
[clients_sahara]
[clients_swift]
[clients_trove]
[cors]
[cors.subdomain]
[database]
connection = *********
[ec2authtoken]
auth_uri = http://172.16.0.11:5000/v2.0/ec2tokens
[eventlet_opts]
[heat_api]
bind_host = 172.16.0.15
workers = 0
[heat_api_cfn]
bind_host = 172.16.0.15
workers = 0
[heat_api_cloudwatch]
bind_host = 172.16.0.15
workers = 0
""".strip()


def test_doc_examples():
    failed_count, tests = doctest.testmod(
        heat_conf,
        globs={'conf': heat_conf.HeatConf(context_wrap(HEAT_CONFIG))}
    )
    assert failed_count == 0


def test_heat_conf():
    h_conf = heat_conf.HeatConf(context_wrap(HEAT_CONFIG))
    assert h_conf.get('DEFAULT', 'heat_metadata_server_url') == 'http://172.16.0.11:8000'
    assert h_conf.get('DEFAULT', 'stack_user_domain_name') == 'heat_stack'
    assert h_conf.get('clients_keystone', 'auth_uri') == 'http://192.0.2.18:35357'
    assert h_conf.get('heat_api_cloudwatch', 'bind_host') == '172.16.0.15'



# Copyright (C) 2013  Jamie Duncan (jduncan@redhat.com)

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# File Name : sos-gov.py
# Creation Date : 10-01-2013
# Created By : Jamie Duncan
# Last Modified : Sat 13 Sep 2014 10:51:54 PM EDT
# Purpose : an sosreport scrubber

import os
import re
import errno
import sys
import uuid
import shutil
import struct, socket
import tempfile
import logging
import tarfile
import six
import hashlib
import json

from insights.util import content_type


class SOSCleaner:
    '''
    A class to parse through an sosreport and begin the cleaning process required in many industries
    Parameters:
    debug - will generate add'l output to STDOUT. defaults to no
    reporting - will post progress and overall statistics to STDOUT. defaults to yes
    '''
    def __init__(self, quiet=False):

        self.name = 'soscleaner'
        self.version = '0.2.2'
        self.loglevel = 'INFO' #this can be overridden by the command-line app
        self.quiet = quiet
        self.domain_count = 0
        self.domains = list()
        self.keywords = list()
        self.domainname = None
        self.report_dir = '/tmp'

        # IP obfuscation information
        self.ip_db = dict() #IP database
        self.start_ip = '10.230.230.1'

        # Hostname obfuscation information
        self.hn_db = dict() #hostname database
        self.hostname_count = 0
        self.hostname = None
        self.fqdn = None
        self.hashed_fqdn = None   # addition for insights-client

        # Domainname obfuscation information
        self.dn_db = dict() #domainname database
        self.root_domain = 'example.com' #right now this needs to be a 2nd level domain, like foo.com, example.com, domain.org, etc.

        # self.origin_path, self.dir_path, self.session, self.logfile, self.uuid = self._prep_environment()
        # self._start_logging(self.logfile)

        # Keyword obfuscation information
        self.keywords = None
        self.kw_db = dict() #keyword database
        self.kw_count = 0

        self.excluded_specs = [
            "insights.specs.Specs.installed_rpms",
            "insights.specs.Specs.dnf_modules",
            "insights.specs.Specs.yum_list_available",
            "insights.specs.Specs.yum_updateinfo",
            "insights.specs.Specs.yum_updates"
        ]

    def _skip_file(self, d, files):
        '''
        The function passed into shutil.copytree to ignore certain patterns and filetypes
        Currently Skipped
        Directories - handled by copytree
        Symlinks - handled by copytree
        Write-only files (stuff in /proc)
        Binaries (can't scan them)
        '''
        skip_list = []
        for f in files:
            f_full = os.path.join(d, f)
            if not os.path.isdir(f_full):
                if not os.path.islink(f_full):
                    #mode = oct(os.stat(f_full).st_mode)[-3:]
                    # executing as root makes this first if clause useless.
                    # i thought i'd already removed it. - jduncan
                    #if mode == '200' or mode == '444' or mode == '400':
                    #    skip_list.append(f)
                    mime_type = content_type.from_file(f_full)
                    if f == 'insights_archive.txt':
                        # don't exclude this file! we need it to parse core collection archives
                        continue
                    if 'text' not in mime_type and 'json' not in mime_type:
                        skip_list.append(f)

        return skip_list

    def _start_logging(self, filename):
        #will get the logging instance going
        loglevel_config = 'logging.%s' % self.loglevel

        #i'd like the stdout to be under another logging name than 'con_out'
        console_log_level = 25  #between INFO and WARNING
        quiet = self.quiet
        logging.addLevelName(console_log_level, "CONSOLE")

        def con_out(self, message, *args, **kws):
            if not quiet:
                self._log(console_log_level, message, args, **kws)

        logging.Logger.con_out = con_out

        logging.basicConfig(filename=filename,
            level=eval(loglevel_config),
            format='%(asctime)s %(name)s %(levelname)s: %(message)s',
            datefmt = '%m-%d %H:%M:%S'
            )
        if not self.quiet: # pragma: no cover
            console = logging.StreamHandler(sys.stdout)
            formatter = logging.Formatter('%(asctime)s %(name)s %(levelname)s: %(message)s', '%m-%d %H:%M:%S')
            console.setFormatter(formatter)
            console.setLevel(console_log_level)
        self.logger = logging.getLogger(__name__)
        if not self.quiet:
            self.logger.addHandler(console) # pragma: no cover

        self.logger.con_out("Log File Created at %s" % filename)

    def _prep_environment(self):

        #we set up our various needed directory structures, etc.
        ran_uuid = str(uuid.uuid4().int)[:16]                                                 # 16 digit random string
        origin_path = os.path.join(self.report_dir, "soscleaner-origin-%s" % ran_uuid)        # the origin dir we'll copy the files into
        dir_path = os.path.join(self.report_dir, "soscleaner-%s" % ran_uuid)                  # the dir we will put our cleaned files into
        session = os.path.join(self.report_dir, "soscleaner-%s" % ran_uuid)                   # short-hand for the soscleaner session to create reports, etc.
        logfile = os.path.join(self.report_dir, "%s.log" % session)                           # the primary logfile

        return origin_path, dir_path, session, logfile, ran_uuid

    def _extract_sosreport(self, path):

        self.logger.con_out("Beginning SOSReport Extraction")
        compression_sig = content_type.from_file(path).lower()
        if 'directory' in compression_sig:
            self.logger.info('%s appears to be a %s - continuing', path, compression_sig)
            # Clear out origin_path as we don't have one
            self.origin_path = None
            return path

        elif 'compressed data' in compression_sig:
            if compression_sig == 'xz compressed data':
                #This is a hack to account for the fact that the tarfile library doesn't
                #handle lzma (XZ) compression until version 3.3 beta
                try:
                    self.logger.info('Data Source Appears To Be LZMA Encrypted Data - decompressing into %s', self.origin_path)
                    self.logger.info('LZMA Hack - Creating %s', self.origin_path)
                    os.system('mkdir %s' % self.origin_path)
                    os.system('tar -xJf %s -C %s' % (path, self.origin_path))
                    return_path = os.path.join(self.origin_path, os.listdir(self.origin_path)[0])

                    return return_path

                except Exception as e: # pragma: no cover
                    self.logger.exception(e)
                    raise Exception('DecompressionError, Unable to decrypt LZMA compressed file %s', path)

            else:
                p = tarfile.open(path, 'r')

                self.logger.info('Data Source Appears To Be %s - decompressing into %s', compression_sig, self.origin_path)
                try:
                    p.extractall(self.origin_path)
                    return_path = os.path.join(self.origin_path, os.path.commonprefix(p.getnames()))

                    return return_path

                except Exception as e:    # pragma: no cover
                    self.logger.exception(e)
                    raise Exception("DeCompressionError: Unable to De-Compress %s into %s", path, self.origin_path)
        else:   # pragma: no cover
            raise Exception('CompressionError: Unable To Determine Compression Type')

    def _sub_ip(self, line):
        '''
        This will substitute an obfuscated IP for each instance of a given IP in a file
        This is called in the self._clean_line function, along with user _sub_* functions to scrub a given
        line in a file.
        It scans a given line and if an IP exists, it obfuscates the IP using _ip2db and returns the altered line
        '''
        try:
            pattern = r"(((\b25[0-5]|\b2[0-4][0-9]|\b1[0-9][0-9]|\b[1-9][0-9]|\b[1-9]))(\.(\b25[0-5]|\b2[0-4][0-9]|\b1[0-9][0-9]|\b[1-9][0-9]|\b[0-9])){3})"
            ips = [each[0] for each in re.findall(pattern, line)]
            if len(ips) > 0:
                for ip in sorted(ips, key=len, reverse=True):
                    # skip loopback (https://github.com/RedHatInsights/insights-core/issues/3230#issuecomment-924859845)
                    if ip != "127.0.0.1" and ip in line:
                        new_ip = self._ip2db(ip)
                        self.logger.debug("Obfuscating IP - %s > %s", ip, new_ip)
                        line = line.replace(ip, new_ip)
            return line
        except Exception as e: # pragma: no cover
            self.logger.exception(e)
            raise Exception('SubIPError: Unable to Substitute IP Address - %s', ip)

    def _sub_ip_netstat(self, line):
        '''
        Special version of _sub_ip for netstat to preserve spacing
        '''
        try:
            pattern = r"(((\b25[0-5]|\b2[0-4][0-9]|\b1[0-9][0-9]|\b[1-9][0-9]|\b[1-9]))(\.(\b25[0-5]|\b2[0-4][0-9]|\b1[0-9][0-9]|\b[1-9][0-9]|\b[0-9])){3})"
            ips = [each[0] for each in re.findall(pattern, line)]
            if len(ips) > 0:
                for ip in sorted(ips, key=len, reverse=True):
                    # skip loopback (https://github.com/RedHatInsights/insights-core/issues/3230#issuecomment-924859845)
                    if ip != "127.0.0.1" and ip in line:
                        ip_len = len(ip)
                        new_ip = self._ip2db(ip)
                        new_ip_len = len(new_ip)

                        self.logger.debug("Obfuscating IP - %s > %s", ip, new_ip)

                        # pad or remove spaces to allow for the new length
                        if ip_len > new_ip_len:
                            numspaces = ip_len - new_ip_len
                            line = line.replace(ip, new_ip)

                            # shift past port specification to add spaces
                            idx = line.index(new_ip) + new_ip_len
                            c = line[idx]
                            while c != " ":
                                idx += 1
                                if idx == len(line):
                                    idx = len(line) - 1
                                    break
                                c = line[idx]
                            line = line[0:idx] + numspaces * " " + line[idx:]

                        elif new_ip_len > ip_len:
                            numspaces = new_ip_len - ip_len
                            line = line.replace(ip, new_ip)

                            # shift past port specification to skip spaces
                            idx = line.index(new_ip) + new_ip_len
                            c = line[idx]
                            while c != " ":
                                idx += 1
                                if idx == len(line):
                                    break
                                c = line[idx]
                            line = line[0:idx] + line[(idx+numspaces):]

                        else:
                            line = line.replace(ip, new_ip)
            return line
        except Exception as e: # pragma: no cover
            self.logger.exception(e)
            raise Exception('SubIPError: Unable to Substitute IP Address - %s', ip)


    def _get_disclaimer(self):  # pragma: no cover
        #prints a disclaimer that this isn't an excuse for manual or any other sort of data verification

        self.logger.con_out("%s version %s" % (self.name, self.version))
        self.logger.warning("%s is a tool to help obfuscate sensitive information from an existing sosreport." % self.name)
        self.logger.warning("Please review the content before passing it along to any third party.")

    def _create_ip_report(self):
        '''
        this will take the obfuscated ip and hostname databases and output csv files
        '''
        try:
            ip_report_name = os.path.join(self.report_dir, "%s-ip.csv" % self.session)
            self.logger.con_out('Creating IP Report - %s', ip_report_name)
            ip_report = open(ip_report_name, 'wt')
            ip_report.write('Obfuscated IP,Original IP\n')
            for k,v in self.ip_db.items():
                ip_report.write('%s,%s\n' %(self._int2ip(k),self._int2ip(v)))
            ip_report.close()
            self.logger.info('Completed IP Report')

            self.ip_report = ip_report_name
        except Exception as e: # pragma: no cover
            self.logger.exception(e)
            raise Exception('CreateReport Error: Error Creating IP Report')

    def _create_hn_report(self):
        try:
            hn_report_name = os.path.join(self.report_dir, "%s-hostname.csv" % self.session)
            self.logger.con_out('Creating Hostname Report - %s', hn_report_name)
            hn_report = open(hn_report_name, 'wt')
            hn_report.write('Obfuscated Hostname,Original Hostname\n')
            if self.hostname_count > 0:
                for k,v in self.hn_db.items():
                    hn_report.write('%s,%s\n' %(k,v))
            else:
                hn_report.write('None,None\n')
            hn_report.close()
            self.logger.info('Completed Hostname Report')

            self.hn_report = hn_report_name
        except Exception as e: #pragma: no cover
            self.logger.exception(e)
            raise Exception('CreateReport Error: Error Creating Hostname Report')

    def _create_dn_report(self):
        try:
            dn_report_name = os.path.join(self.report_dir, "%s-dn.csv" % self.session)
            self.logger.con_out('Creating Domainname Report - %s', dn_report_name)
            dn_report = open(dn_report_name, 'wt')
            dn_report.write('Obfuscated Domain,Original Domain\n')
            if self.domain_count > 0:
                for k,v in self.dn_db.items():
                    dn_report.write('%s,%s\n' %(k,v))
            else:
                dn_report.write('None,None\n')
            dn_report.close()
            self.logger.info('Completed Domainname Report')

            self.dn_report = dn_report_name

        except Exception as e: # pragma: no cover
            self.logger.exception(e)
            raise Exception('CreateReport Error: Error Creating Domainname Report')

    def _create_reports(self): # pragma: no cover

        self._create_ip_report()
        self._create_hn_report()
        self._create_dn_report()

    def _sub_hostname(self, line):
        '''
        This will replace the exact hostname and all instances of the domain name with the obfuscated alternatives.
        Example:
        '''
        try:
            for od,d in self.dn_db.items():
                #regex = re.compile(r'\w*\.%s' % d)
                regex = re.compile(r'(?![\W\-\:\ \.])[a-zA-Z0-9\-\_\.]*\.%s' % d)
                hostnames = [each for each in regex.findall(line)]
                if len(hostnames) > 0:
                    for hn in hostnames:
                        new_hn = self._hn2db(hn)
                        self.logger.debug("Obfuscating FQDN - %s > %s", hn, new_hn)
                        line = line.replace(hn, new_hn)
            if self.hostname:
                line = line.replace(self.hostname, self._hn2db(self.fqdn))  #catch any non-fqdn instances of the system hostname

            return line
        except Exception as e: # pragma: no cover
            self.logger.exception(e)
            raise Exception('SubHostnameError: Unable to Substitute Hostname/Domainname')

    def _make_dest_env(self):
        '''
        This will create the folder in self.report_dir (defaults to /tmp) to store the sanitized files and populate it using shutil
        These are the files that will be scrubbed
        '''
        try:
            shutil.copytree(self.report, self.dir_path, symlinks=True, ignore=self._skip_file)

        except Exception as e:    #pragma: no cover
            self.logger.exception(e)
            raise Exception("DestinationEnvironment Error: Cannot Create Destination Environment")

    def _create_archive(self):
        '''This will create a tar.gz compressed archive of the scrubbed directory'''
        try:
            self.archive_path = os.path.join(self.report_dir, "%s.tar.gz" % self.session)
            self.logger.con_out('Creating SOSCleaner Archive - %s', self.archive_path)
            t = tarfile.open(self.archive_path, 'w:gz')
            for dirpath, dirnames, filenames in os.walk(self.dir_path):
                for f in filenames:
                    f_full = os.path.join(dirpath, f)
                    f_archive = f_full.replace(self.report_dir,'')
                    self.logger.debug('adding %s to %s archive', f_archive, self.archive_path)
                    t.add(f_full, arcname=f_archive)
        except Exception as e: #pragma: no cover
            self.logger.exception(e)
            raise Exception('CreateArchiveError: Unable to create Archive')

        self._clean_up()
        self.logger.info('Archiving Complete')
        self.logger.con_out('SOSCleaner Complete')
        if not self.quiet:  # pragma: no cover
            t.add(self.logfile, arcname=self.logfile.replace(self.report_dir,''))
        t.close()

    def _clean_up(self):
        '''This will clean up origin directories, etc.'''
        self.logger.info('Beginning Clean Up Process')
        try:
            if self.origin_path:
                self.logger.info('Removing Origin Directory - %s', self.origin_path)
                shutil.rmtree(self.origin_path)
            self.logger.info('Removing Working Directory - %s', self.dir_path)
            shutil.rmtree(self.dir_path)
            self.logger.info('Clean Up Process Complete')
        except Exception as e:    #pragma: no cover
            self.logger.exception(e)

    def _process_hosts_file(self, options):
        # this will process the hosts file more thoroughly to try and capture as many server short names/aliases as possible
        # could lead to false positives if people use dumb things for server aliases, like 'file' or 'server' or other common terms
        # this may be an option that can be enabled... --hosts or similar?

        if options.core_collect:
            hosts_file = os.path.join(self.dir_path, 'data', 'etc/hosts')
        else:
            hosts_file = os.path.join(self.dir_path, 'etc/hosts')

        try:
            if os.path.isfile(hosts_file):
                with open(hosts_file) as f:
                    self.logger.con_out("Processing hosts file for better obfuscation coverage")
                    data = f.readlines()
                    for line in data:
                        x = re.split('\ |\t', line.rstrip())    #chunk up the line, delimiting with spaces and tabs (both used in hosts files)
                        # we run through the rest of the items in a given line, ignoring the IP to be picked up by the normal methods
                        # skipping over the 'localhost' and 'localdomain' entries
                        for item in x[1:len(x)]:
                            if len(item) > 0:
                                if all(['localhost' not in item, 'localdomain' not in item]):
                                    new_host = self._hn2db(item)
                                    self.logger.debug("Added to hostname database through hosts file processing - %s > %s", item, new_host)
            else: # pragma: no cover
                self.logger.con_out("Unable to Process Hosts File. Hosts File Processing Disabled")

        except Exception as e:    #pragma: no cover
            self.logger.exception(e)

    def _domains2db(self):
        #adds any additional domainnames to the domain database to be searched for
        try:
            #we will add the root domain for an FQDN as well.
            if self.domainname is not None:
                self.dn_db[self.root_domain] = self.domainname
                self.logger.con_out("Obfuscated Domain Created - %s" % self.root_domain)

            split_root_d = self.root_domain.split('.')

            for d in self.domains:
                if d not in self.dn_db.values(): #no duplicates
                    d_number = len(self.dn_db)
                    o_domain = "%s%s.%s" % (split_root_d[0], d_number, split_root_d[1])
                    self.dn_db[o_domain] = d
                    self.logger.con_out("Obfuscated Domain Created - %s" % o_domain)

            self.domain_count = len(self.dn_db)
            return True

        except Exception as e: # pragma: no cover
            self.logger.exception(e)

    def _keywords2db(self):
        #processes optional keywords to add to be obfuscated
        try:
            if self.keywords:   # value is set to None by default
                k_count = 0
                for f in self.keywords:
                    if os.path.isfile(f):
                        with open(f, 'rt') as klist:
                            for keyword in klist.readlines():
                                o_kw = "keyword%s" % k_count
                                self.kw_db[keyword.rstrip()] = o_kw
                                self.logger.con_out("Added Obfuscated Keyword - %s", o_kw)
                                k_count += 1
                        self.logger.con_out("Added Keyword Contents from file - %s", f)

                    else:
                        self.logger.con_out("%s does not seem to be a file. Not adding any keywords from" % f)

            self.kw_count = k_count

        except Exception as e: # pragma: no cover
            self.logger.exception(e)

    def _kw2db(self, keyword):
        #returns the obfuscated value for a keyword

        return self.kw_db[keyword]

    def _sub_keywords(self, line):
        # this will substitute out any keyword entries on a given line
        #try:
        if self.kw_count > 0:    # we have obfuscated keywords to work with
            for k in self.kw_db.keys():
                if k in line:
                    line = line.replace(k, self._kw2db(k))
                    self.logger.debug("Obfuscating Keyword - %s > %s", k, self._kw2db(k))

        return line

        '''except Exception, e: # pragma: no cover
            self.logger.exception(e)
            raise Exception('SubKeywordError: Unable to Substitute Keywords')'''

    def _get_hostname(self, hostname='hostname'):
        #gets the hostname and stores hostname/domainname so they can be filtered out later

        try:
            hostfile = os.path.join(self.dir_path, hostname)
            fh = open(hostfile, 'rt')
            name_list = fh.readline().rstrip().split('.')
            hostname = name_list[0]
            fqdn = '.'.join(name_list)  # insights-client needs FQDN
            if len(name_list) > 1:
                domainname = '.'.join(name_list[1:len(name_list)])
            else:
                domainname = None

            return hostname, domainname, fqdn

        except IOError as e: #the 'hostname' file doesn't exist or isn't readable for some reason
            self.logger.warning("Unable to determine system hostname!!!")
            self.logger.warning("Automatic Hostname Data Obfuscation Will Not Occur!!!")
            self.logger.warning("To Remedy This Situation please enable the 'general' plugin when running sosreport")
            self.logger.warning("and/or be sure the 'hostname' symlink exists in the root directory of you sosreport")
            if not self.quiet:
                self.logger.exception(e)

            hostname = None
            domainname = None
            fqdn = None

            return hostname, domainname, fqdn

        except Exception as e: # pragma: no cover
            self.logger.exception(e)
            raise Exception('GetHostname Error: Cannot resolve hostname from %s') % hostfile

    def _ip2int(self, ipstr):
        #converts a dotted decimal IP address into an integer that can be incremented
        integer = struct.unpack('!I', socket.inet_aton(ipstr))[0]

        return integer

    def _int2ip(self, num):
        #converts an integer stored in the IP database into a dotted decimal IP
        ip = socket.inet_ntoa(struct.pack('!I', num))

        return ip

    def _ip2db(self, ip):
        '''
        adds an IP address to the IP database and returns the obfuscated entry, or returns the
        existing obfuscated IP entry
        FORMAT:
        {$obfuscated_ip: $original_ip,}
        '''

        ip_num = self._ip2int(ip)
        ip_found = False
        db = self.ip_db
        for k,v in db.items():
            if v == ip_num:
                ret_ip = self._int2ip(k)
                ip_found = True
        if ip_found:                #the entry already existed
            return ret_ip
        else:                       #the entry did not already exist
            if len(self.ip_db) > 0:
                new_ip = max(db.keys()) + 1
            else:
                new_ip = self._ip2int(self.start_ip)
            db[new_ip] = ip_num

            return self._int2ip(new_ip)

    def _hn2db(self, hn):
        '''
        This will add a hostname for a hostname for an included domain or return an existing entry
        '''
        db = self.hn_db
        hn_found = False
        for k,v in db.items():
            if v == hn:  #the hostname is in the database
                ret_hn = k
                hn_found = True
        if hn_found:
            return ret_hn
        else:
            self.hostname_count += 1    #we have a new hostname, so we increment the counter to get the host ID number
            o_domain = self.root_domain
            for od,d in self.dn_db.items():
                if d in hn:
                    o_domain = od
            new_hn = "host%s.%s" % (self.hostname_count, o_domain)
            self.hn_db[new_hn] = hn

            return new_hn

    def _walk_report(self, folder):
        '''returns a dictonary of dictionaries in the format {directory_name:[file1,file2,filex]}'''

        dir_list = {}
        try:
            for dirName, subdirList, fileList in os.walk(folder):
                x = []
                for fname in fileList:
                    x.append(fname)
                dir_list[dirName] = x

            return dir_list
        except Exception as e: # pragma: no cover
            self.logger.exception(e)
            raise Exception("WalkReport Error: Unable to Walk Report")

    def _file_list(self, folder):
        '''returns a list of file names in an sosreport directory'''
        rtn = []
        walk = self._walk_report(folder)
        for key,val in walk.items():
            for v in val:
                x=os.path.join(key,v)
                rtn.append(x)

        self.file_count = len(rtn)  #a count of the files we'll have in the final cleaned sosreport, for reporting
        return rtn

    def _clean_line(self, l, f=None):
        '''this will return a line with obfuscations for all possible variables, hostname, ip, etc.'''

        if f and f.endswith("netstat_-neopa"):
            new_line = self._sub_ip_netstat(l)                  # IP substitution
        else:
            new_line = self._sub_ip(l)                  # IP substitution
        new_line = self._sub_hostname(new_line)     # Hostname substitution
        new_line = self._sub_keywords(new_line)     # Keyword Substitution

        return new_line

    def _clean_file(self, f):
        '''this will take a given file path, scrub it accordingly, and save a new copy of the file
        in the same location'''
        if os.path.exists(f) and not os.path.islink(f):
            tmp_file = tempfile.TemporaryFile(mode='w+b')
            try:
                fh = open(f, 'r')
                data = fh.readlines()
                fh.close()
                if len(data) > 0: #if the file isn't empty:
                    for l in data:
                        new_l = self._clean_line(l, f)
                        if six.PY3:
                            tmp_file.write(new_l.encode('utf-8'))
                        else:
                            tmp_file.write(new_l)

                    tmp_file.seek(0)

            except Exception as e: # pragma: no cover
                self.logger.exception(e)
                raise Exception("CleanFile Error: Cannot Open File For Reading - %s" % f)

            try:
                if len(data) > 0:
                    new_fh = open(f, 'wb')
                    for line in tmp_file:
                        new_fh.write(line)
                    new_fh.close()
            except Exception as e: # pragma: no cover
                self.logger.exception(e)
                raise Exception("CleanFile Error: Cannot Write to New File - %s" % f)

            finally:
                tmp_file.close()

    def _add_extra_files(self, files):
        '''if extra files are to be analyzed with an sosreport, this will add them to the origin path to be analyzed'''

        try:
            for f in files:
                self.logger.con_out("adding additional file for analysis: %s"  % f)
                fname = os.path.basename(f)
                f_new = os.path.join(self.dir_path, fname)
                shutil.copyfile(f,f_new)
        except IOError as e:
            self.logger.con_out("ExtraFileError: %s is not readable or does not exist. Skipping File" % f)
            self.logger.exception(e)
            pass
        except Exception as e:    # pragma: no cover
            self.logger.exception(e)
            raise Exception("ExtraFileError: Unable to Process Extra File - %s" % f)

    def _clean_files_only(self, files):
        ''' if a user only wants to process one or more specific files, instead of a full sosreport '''
        try:
            if not (os.path.exists(self.origin_path)):
                self.logger.info("Creating Origin Path - %s" % self.origin_path)
                os.makedirs(self.origin_path) # create the origin_path directory
            if not (os.path.exists(self.dir_path)):
                self.logger.info("Creating Directory Path - %s" % self.dir_path)
                os.makedirs(self.dir_path)    # create the dir_path directory
            self._add_extra_files(files)

        except OSError as e: # pragma: no cover
            if e.errno == errno.EEXIST:
                pass
            else:   # pragma: no cover
                self.logger.exception(e)
                raise e

        except Exception as e:    # pragma: no cover
            self.logger.exception(e)
            raise Exception("CleanFilesOnlyError: unable to process")

    def _excluded_files(self):
        '''Collect data files excluded from IP address obfuscation.
        Their paths are part of the meta_data JSONs.
        '''
        excluded_files = []
        meta_data_path = os.path.join(self.dir_path, "meta_data")
        for dir_name, file_names in self._walk_report(meta_data_path).items():
            for file_name in file_names:
                file_path = os.path.join(dir_name, file_name)
                with open(file_path) as file:
                    meta_data = json.load(file)
                if meta_data["name"] in self.excluded_specs:
                    if isinstance(meta_data["results"], list):
                        results = meta_data["results"]
                    else:
                        results = [meta_data["results"]]
                    relative_paths = [result["object"]["relative_path"] for result in results if result and 'object' in result]
                    excluded_files.extend(relative_paths) if relative_paths else None
        return excluded_files

    def clean_report(self, options, sosreport): # pragma: no cover
        '''this is the primary function, to put everything together and analyze an sosreport'''

        if options.report_dir: # override the default location for artifacts (/tmp)
            if os.path.isdir(options.report_dir):
                self.report_dir = options.report_dir
        self.origin_path, self.dir_path, self.session, self.logfile, self.uuid = self._prep_environment()
        self._start_logging(self.logfile)
        self._get_disclaimer()
        if options.domains:
            self.domains = options.domains
        if options.keywords:
            self.keywords = options.keywords
            self._keywords2db()
        if not sosreport:
            if not options.files:
                raise Exception("Error: You must supply either an sosreport and/or files to process")

            self.logger.con_out("No sosreport supplied. Only processing specific files")
            self._clean_files_only(options.files)

        else:   # we DO have an sosreport to analyze
            self.report = self._extract_sosreport(sosreport)
            self._make_dest_env()   # create the working directory
            if options.hostname_path:
                self.hostname, self.domainname, self.fqdn = self._get_hostname(options.hostname_path)
            else:
                self.hostname, self.domainname, self.fqdn = self._get_hostname()

            if options.files:
                self._add_extra_files(options.files)

            if self.fqdn:   # if we have a hostname that's not a None type
                if six.PY3:
                    self.hashed_fqdn = hashlib.sha1(self.fqdn.encode('utf-8')).hexdigest() + '.example.com'
                else:
                    self.hashed_fqdn = hashlib.sha1(self.fqdn).hexdigest() + '.example.com'
                self.hn_db[self.hashed_fqdn] = self.fqdn     # we'll prime the hostname pump to clear out a ton of useless logic later

            self._process_hosts_file(options)  # we'll take a dig through the hosts file and make sure it is as scrubbed as possible

        self._domains2db()

        data_path = os.path.join(self.dir_path, 'data') if options.core_collect else self.dir_path
        files = self._file_list(data_path)

        excluded_files = []
        for f in self._excluded_files():
            f_path = os.path.join(data_path, f)
            excluded_files.append(f_path)

        self.logger.con_out("IP Obfuscation Start Address - %s", self.start_ip)
        self.logger.con_out("*** SOSCleaner Processing ***")
        self.logger.info("Working Directory - %s", self.dir_path)
        for f in files:
            if f in excluded_files:
                self.logger.debug("File %s excluded from IP obfuscation.", f)
                continue

            if options.core_collect:
                # set a relative path of $ARCHIVEROOT/data for core collection
                relative_path = os.path.relpath(f, start=os.path.join(self.dir_path, 'data'))
            else:
                # set a relative path of $ARCHIVEROOT for non core collection
                relative_path = os.path.relpath(f, start=self.dir_path)

                # in addition to setting up that relative path, skip these
                #  files in the archive root for classic collection
                if relative_path in ('display_name',
                                     'blacklist_report',
                                     'tags.json',
                                     'branch_info',
                                     'version_info',
                                     'egg_release'):
                    continue
            # ALWAYS skip machine-id, subman id, and insights id
            if relative_path in ('etc/machine-id',
                                 'etc/insights-client/machine-id'):
                continue
            self.logger.debug("Cleaning %s", f)
            self._clean_file(f)
        self.logger.con_out("*** SOSCleaner Statistics ***")
        self.logger.con_out("IP Addresses Obfuscated - %s", len(self.ip_db))
        self.logger.con_out("Hostnames Obfuscated - %s" , len(self.hn_db))
        self.logger.con_out("Domains Obfuscated - %s" , len(self.dn_db))
        self.logger.con_out("Total Files Analyzed - %s", self.file_count)
        self.logger.con_out("*** SOSCleaner Artifacts ***")
        self._create_reports()

        # This if-block added for insights-client --output-dir option.
        #   Prevent calling of _create_archive() so that dir_path is
        #   not removed and archive is not unnecessarily created.
        #   Furthermore, the client does not utilize soscleaner return
        #   data, so return here. Further code beyond the block
        #   is left as-is.
        if options.no_tar_file:
            return
        # end insights-client modifications

        self._create_archive()
        return_data = [self.archive_path, self.logfile, self.ip_report]

        if self.hostname:
            return_data.append(self.hn_report)
        if len(self.dn_db) >= 1:
            return_data.append(self.dn_report)

        return return_data



from insights.parsers.logrotate_conf import LogrotateConf
from insights.combiners import logrotate_conf
from insights.combiners.logrotate_conf import LogrotateConfAll
from insights.tests import context_wrap
import doctest

LOGROTATE_CONF_1 = """
# see "man logrotate" for details
# rotate log files weekly
weekly

# keep 4 weeks worth of backlogs
rotate 4

# create new (empty) log files after rotating old ones
create

# use date as a suffix of the rotated file
dateext

# uncomment this if you want your log files compressed
#compress

# RPM packages drop log rotation information into this directory
include /etc/logrotate.d

# no packages own wtmp and btmp -- we'll rotate them here
/var/log/wtmp {
    monthly
    create 0664 root utmp
        minsize 1M
    rotate 1
}
""".strip()

LOGROTATE_CONF_2 = """
/var/log/candlepin/*.log {
    copytruncate
    daily
    rotate 52
    compress
    missingok
    create 0644 tomcat tomcat
}
""".strip()

LOGROTATE_CONF_3 = """
/var/log/cron
/var/log/maillog
/var/log/messages
/var/log/secure
/var/log/spooler
{
    sharedscripts
    postrotate
        /bin/kill -HUP `cat /var/run/syslogd.pid 2> /dev/null` 2> /dev/null || true
    endscript
}
""".strip()

LOGROTATE_CONF_NO_FILE = """
# use date as a suffix of the rotated file
dateext

# RPM packages drop log rotation information into this directory
include /etc/logrotate.d
""".strip()

LOGROTATE_MAN_PAGE_DOC_1 = """
# sample file
compress
rotate 7

/var/log/messages {
    rotate 5
    weekly
    postrotate
                /sbin/killall -HUP syslogd
    endscript
}
""".strip()

LOGROTATE_MAN_PAGE_DOC_2 = """
"/var/log/httpd/access.log" /var/log/httpd/error.log {
    rotate 5
    mail www@my.org
    size=100k
    sharedscripts
    postrotate
                /sbin/killall -HUP httpd
    endscript
}
""".strip()

LOGROTATE_MAN_PAGE_DOC_3 = """
/var/log/news/*.crit
{
    monthly
    rotate 2
    olddir /var/log/news/old
    missingok
    postrotate
                kill -HUP `cat /var/run/inn.pid`
    endscript
    nocompress
}
""".strip()


def test_web_xml_doc_examples():
    env = {
            'all_lrt': LogrotateConfAll(
                [
                    LogrotateConf(context_wrap(LOGROTATE_MAN_PAGE_DOC_1, path='/etc/logrotate.conf')),
                    LogrotateConf(context_wrap(LOGROTATE_MAN_PAGE_DOC_2, path='/etc/logrotate.d/httpd')),
                    LogrotateConf(context_wrap(LOGROTATE_MAN_PAGE_DOC_3, path='/etc/logrotate.d/newscrit'))
                ])
          }
    failed, total = doctest.testmod(logrotate_conf, globs=env)
    assert failed == 0


def test_logrotate_conf_combiner():
    all_lrt = LogrotateConfAll(
            [
                LogrotateConf(context_wrap(LOGROTATE_CONF_1, path='/etc/logrotate.conf')),
                LogrotateConf(context_wrap(LOGROTATE_CONF_2, path='/etc/logrotate.d/candlepin')),
                LogrotateConf(context_wrap(LOGROTATE_CONF_3, path='/etc/logrotate.d/xx'))
            ])
    assert all_lrt.global_options == ['weekly', 'rotate', 'create', 'dateext', 'include']
    assert all_lrt['rotate'] == '4'
    assert '/var/log/httpd/access.log' not in all_lrt
    assert all_lrt['/var/log/httpd/access.log'] is None
    assert all_lrt.options_of_logfile('/var/log/httpd/access.log') is None
    assert all_lrt.configfile_of_logfile('/var/log/httpd/access.log') is None
    assert '/var/log/candlepin/error.log' in all_lrt
    assert all_lrt['/var/log/candlepin/candlepin.log']['rotate'] == '52'
    assert all_lrt.configfile_of_logfile('/var/log/wtmp') == '/etc/logrotate.conf'
    assert all_lrt.configfile_of_logfile('/var/log/candlepin/access.log') == '/etc/logrotate.d/candlepin'
    assert all_lrt.configfile_of_logfile('/var/log/maillog') == '/etc/logrotate.d/xx'
    assert all_lrt.options_of_logfile('/var/log/maillog')['sharedscripts'] is True


def test_logrotate_conf_combiner_no_logfile():
    all_lrt = LogrotateConfAll(
            [
                LogrotateConf(context_wrap(LOGROTATE_CONF_NO_FILE, path='/etc/logrotate.conf')),
            ])
    assert all_lrt.global_options == ['dateext', 'include']
    assert all_lrt.log_files == []
    assert all_lrt.options_of_logfile('/var/log/httpd/access.log') is None
    assert all_lrt.configfile_of_logfile('/var/log/httpd/access.log') is None



import pytest

from insights.core.exceptions import ParseException
from insights.parsers.zipl_conf import ZiplConf
from insights.tests import context_wrap

ZIPL_CONF = """
[defaultboot]
defaultauto
prompt=1
timeout=5
default=linux
target=/boot
[linux]
    image=/boot/vmlinuz-3.10.0-693.el7.s390x
    ramdisk=/boot/initramfs-3.10.0-693.el7.s390x.img
    parameters="root=/dev/mapper/rhel_gss5-root crashkernel=auto rd.dasd=0.0.0100 rd.dasd=0.0.0101 rd.dasd=0.0.0102 rd.lvm.lv=rhel_gss5/root rd.lvm.lv=rhel_gss5/swap net.ifnames=0 rd.znet=qeth,0.0.0600,0.0.0601,0.0.0602,layer2=0,portname=gss5,portno=0 LANG=en_US.UTF-8"
[linux-0-rescue-a27932c8d57248e390cee3798bbd3709]
    image=/boot/vmlinuz-0-rescue-a27932c8d57248e390cee3798bbd3709
    ramdisk=/boot/initramfs-0-rescue-a27932c8d57248e390cee3798bbd3709.img
    parameters="root=/dev/mapper/rhel_gss5-root crashkernel=auto rd.dasd=0.0.0100 rd.dasd=0.0.0101 rd.dasd=0.0.0102 rd.lvm.lv=rhel_gss5/root rd.lvm.lv=rhel_gss5/swap net.ifnames=0 rd.znet=qeth,0.0.0600,0.0.0601,0.0.0602,layer2=0,portname=gss5,portno=0"
[other]
    image=/boot/vmlinuz
    ramdisk=/boot/initramfs.img
    parameters="root=/dev/mapper/rhel_gss5-root crashkernel=auto rd.dasd=0.0.0100

# Configuration for dumping to SCSI disk
# Separate IPL and dump partitions
[dumpscsi]
target=/boot
dumptofs=/dev/sda2
parameters="dump_dir=/mydumps dump_compress=none dump_mode=auto"

# Menu containing two DASD boot configurations
:menu1
1=linux
2=linux-0-rescue-a27932c8d57248e390cee3798bbd3709
default=1
prompt=1
timeout=30
""".strip()

ZIPL_CONF_INVALID = """
prompt=1
timeout=5
default=linux
[linux]
    image=/boot/vmlinuz-3.10.0-693.el7.s390x
    ramdisk=/boot/initramfs-3.10.0-693.el7.s390x.img
    parameters="root=/dev/mapper/rhel_gss5-root crashkernel=auto rd.dasd=0.0.0100 rd.dasd=0.0.0101 rd.dasd=0.0.0102 rd.lvm.lv=rhel_gss5/root rd.lvm.lv=rhel_gss5/swap net.ifnames=0 rd.znet=qeth,0.0.0600,0.0.0601,0.0.0602,layer2=0,portname=gss5,portno=0 LANG=en_US.UTF-8"
""".strip()


def test_zipl_conf():
    res = ZiplConf(context_wrap(ZIPL_CONF))
    assert res.get('linux').get('image') == "/boot/vmlinuz-3.10.0-693.el7.s390x"
    assert res['linux']['image'] == "/boot/vmlinuz-3.10.0-693.el7.s390x"
    assert res[':menu1']['1'] == 'linux'
    assert 'defaultauto' in res['defaultboot']
    assert res['defaultboot']['defaultauto'] is True
    assert res['other']['parameters'] == '"root=/dev/mapper/rhel_gss5-root crashkernel=auto rd.dasd=0.0.0100'
    assert res.images == {
                            'linux': '/boot/vmlinuz-3.10.0-693.el7.s390x',
                            'linux-0-rescue-a27932c8d57248e390cee3798bbd3709': '/boot/vmlinuz-0-rescue-a27932c8d57248e390cee3798bbd3709',
                            'other': '/boot/vmlinuz'
                        }
    assert res.dumptofses == {'dumpscsi': '/dev/sda2'}


def test_zipl_conf_invalid():
    with pytest.raises(ParseException) as pe:
        ZiplConf(context_wrap(ZIPL_CONF_INVALID))
    assert "Invalid zipl configuration file is found." in str(pe)



from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider
from insights.client import auto_config
from insights.client.apps.malware_detection import MalwareDetectionClient
from insights.client.config import InsightsConfig
from insights.specs import Specs
import logging
logger = logging.getLogger(__name__)


class MalwareDetectionSpecs(Specs):
    # timeout=0 disables the datasource timeout alarm, allowing malware-detection to run for as long as necessary
    @datasource(HostContext, timeout=0)
    def malware_detection_app(broker):
        """
        Custom datasource to collects content for malware scanner if a scanner is present on the system
        """

        try:
            # Only run malware-detection if it was passed as an option to insights-client
            insights_config = InsightsConfig().load_all()
            auto_config.try_auto_configuration(insights_config)
            if not (insights_config and hasattr(insights_config, 'app') and insights_config.app == 'malware-detection'):
                raise SkipComponent("Only run malware-detection app when specifically requested via --collector option")
            mdc = MalwareDetectionClient(insights_config)
            scan_results = mdc.run()
            if scan_results:
                return DatasourceProvider(content=scan_results, relative_path="malware-detection-results.json")
            else:
                raise SkipComponent("No scan results were produced")
        except SkipComponent as msg:
            raise SkipComponent("Skipping malware-detection app: {0}".format(str(msg)))
        except Exception as err:
            from traceback import format_exc
            err_msg = "Unexpected exception in malware-detection app: {0}".format(str(err))
            logger.error(err_msg)
            logger.debug(format_exc())
            raise SkipComponent(err_msg)



import doctest
from datetime import datetime
from insights.parsers import nginx_log
from insights.parsers.nginx_log import ContainerNginxErrorLog, NginxErrorLog
from insights.tests import context_wrap


NGINX_ERROR_LOG = """
2022/04/02 04:07:59 [warn] 1591#1591: *697425 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/2/25/0000003252 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=46 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "towergtd.desjardins.com"
2022/04/02 04:08:46 [warn] 1592#1592: *697439 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/3/25/0000003253 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=50 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "towergtd.desjardins.com"
2022/04/02 04:09:27 [warn] 1593#1593: *697455 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/4/25/0000003254 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=53 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "towergtd.desjardins.com"
2022/04/02 05:30:59 [warn] 1591#1591: *698917 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/5/25/0000003255 while reading upstream, client: ::1, server: _, request: "GET /api/v2/inventories/61/script/?hostvars=1&towervars=1&all=1 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "localhost:44443"
2022/04/02 17:59:29 [warn] 1594#1594: *711881 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/6/25/0000003256 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/labels/?page_size=200 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "towergtd.desjardins.com", referrer: "https://towergtd.desjardins.com/"
2022/04/03 00:02:28 [warn] 1591#1591: *719136 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/7/25/0000003257 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/organizations/?page_size=999 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "towergtd.desjardins.com"
""".strip()


def test_nginx_error_log():
    error_log = NginxErrorLog(context_wrap(NGINX_ERROR_LOG))
    assert len(error_log.lines) == 6
    assert "1591#1591: *698917" in error_log
    assert error_log.lines[0] == '2022/04/02 04:07:59 [warn] 1591#1591: *697425 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/2/25/0000003252 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=46 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "towergtd.desjardins.com"'
    assert len(list(error_log.get_after(datetime(2022, 4, 2, 5, 0, 0)))) == 3


def test_container_nginx_error_log():
    container_error_log = ContainerNginxErrorLog(
        context_wrap(
            NGINX_ERROR_LOG,
            container_id='2869b4e2541c',
            image='registry.access.redhat.com/ubi8/nginx-120',
            engine='podman',
            path='insights_containers/2869b4e2541c/var/log/nginx/error.log'
        )
    )
    assert container_error_log.image == "registry.access.redhat.com/ubi8/nginx-120"
    assert container_error_log.engine == "podman"
    assert container_error_log.container_id == "2869b4e2541c"
    assert len(container_error_log.lines) == 6
    assert "1591#1591: *698917" in container_error_log
    assert container_error_log.lines[0] == '2022/04/02 04:07:59 [warn] 1591#1591: *697425 an upstream response is buffered to a temporary file /var/lib/nginx/tmp/uwsgi/2/25/0000003252 while reading upstream, client: 10.245.136.148, server: _, request: "GET /api/v2/hosts/?not__name=localhost&page_size=400&page=46 HTTP/1.1", upstream: "uwsgi://unix:/var/run/tower/uwsgi.sock:", host: "towergtd.desjardins.com"'
    assert len(list(container_error_log.get_after(datetime(2022, 4, 2, 5, 0, 0)))) == 3


def test_doc():
    env = {
        "nginx_error_log": NginxErrorLog(context_wrap(NGINX_ERROR_LOG)),
        "container_nginx_error_log": ContainerNginxErrorLog(
            context_wrap(
                NGINX_ERROR_LOG,
                container_id='2869b4e2541c',
                image='registry.access.redhat.com/ubi8/nginx-120',
                engine='podman'
            )
        )
    }
    failed_count, total = doctest.testmod(nginx_log, globs=env)
    assert failed_count == 0



import doctest
import pytest

from insights.combiners import cloud_instance
from insights.combiners.cloud_provider import CloudProvider
from insights.combiners.cloud_instance import CloudInstance
from insights.core.exceptions import ContentException, SkipComponent
from insights.parsers.aws_instance_id import AWSInstanceIdDoc
from insights.parsers.azure_instance import AzureInstanceID, AzureInstanceType
from insights.parsers.gcp_instance_type import GCPInstanceType
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.subscription_manager import SubscriptionManagerFacts
from insights.tests import context_wrap
from insights.tests.combiners.test_cloud_provider import RPMS_AWS, RPMS_GOOGLE, RPMS_AZURE
from insights.tests.parsers.test_aws_instance_id import AWS_ID_DOC
from insights.tests.parsers.test_azure_instance import AZURE_ID, AZURE_TYPE_2
from insights.tests.parsers.test_gcp_instance_type import GOOGLE_TYPE_1
from insights.tests.parsers.test_subscription_manager import INPUT_NORMAL_1

GOOGLE_RHSM_FACTS = """
gcp_instance_id: 567890567890
network.ipv6_address: ::1
uname.sysname: Linux
""".strip()


def test_cloud_instance_google():
    rpms = InstalledRpms(context_wrap(RPMS_GOOGLE))
    _type = GCPInstanceType(context_wrap(GOOGLE_TYPE_1))
    cp = CloudProvider(rpms, None, None, None)
    facts = SubscriptionManagerFacts(context_wrap(GOOGLE_RHSM_FACTS))
    ret = CloudInstance(cp, None, None, None, _type, facts)
    assert ret.provider == CloudProvider.GOOGLE
    assert ret.id == "567890567890"
    assert ret.type == "n2-highcpu-16"
    assert ret.size == "n2-highcpu-16"


def test_cloud_instance_aws():
    rpms = InstalledRpms(context_wrap(RPMS_AWS))
    _id = AWSInstanceIdDoc(context_wrap(AWS_ID_DOC))
    cp = CloudProvider(rpms, None, None, None)
    ret = CloudInstance(cp, _id, None, None, None, None)
    assert ret.provider == CloudProvider.AWS
    assert ret.id == "i-1234567890abcdef0"
    assert ret.type == "t2.micro"
    assert ret.size == "t2.micro"


def test_cloud_instance_azure():
    rpms = InstalledRpms(context_wrap(RPMS_AZURE))
    _id = AzureInstanceID(context_wrap(AZURE_ID))
    _type = AzureInstanceType(context_wrap(AZURE_TYPE_2))
    cp = CloudProvider(rpms, None, None, None)
    ret = CloudInstance(cp, None, _id, _type, None, None)
    assert ret.provider == CloudProvider.AZURE
    assert ret.id == "f904ece8-c6c1-4b5c-881f-309b50f25e50"
    assert ret.type == "Standard_NV48s_v3"
    assert ret.size == "Standard_NV48s_v3"


def test_cloud_instance_ex():
    rpms = InstalledRpms(context_wrap(RPMS_GOOGLE))
    _type = GCPInstanceType(context_wrap(GOOGLE_TYPE_1))
    cp = CloudProvider(rpms, None, None, None)
    aws_facts = SubscriptionManagerFacts(context_wrap(INPUT_NORMAL_1))

    with pytest.raises(ContentException) as ce:
        CloudInstance(cp, None, None, None, None, aws_facts)
    assert "Unmatched" in str(ce)

    with pytest.raises(SkipComponent):
        CloudInstance(cp, None, None, None, _type, None)


def test_cloud_instance_doc():
    rpms = InstalledRpms(context_wrap(RPMS_AWS))
    _id = AWSInstanceIdDoc(context_wrap(AWS_ID_DOC))
    cp = CloudProvider(rpms, None, None, None)
    env = {'ci': CloudInstance(cp, _id, None, None, None, None)}
    failed, total = doctest.testmod(cloud_instance, globs=env)
    assert failed == 0



"""
CloudCfg - ``/etc/cloud/cloud.cfg``
===================================
Module to parser the content of ``/etc/cloud/cloud.cfg`` file.
"""
from insights import YAMLParser, parser
from insights.specs import Specs


@parser(Specs.cloud_cfg_filtered)
class CloudCfg(YAMLParser):
    """
    This parser parses the ``/etc/cloud/cloud.cfg`` file collected via the
    :mod:`insights.specs.Specs.cloud_cfg_filtered` spec which is the
    filtered per the :mod:`insights.specs.Specs.cloud_cfg` spec into a
    dictionary.

    .. note::
        Since this file may contain sensitive information, it should be
        filtered before the Insights collecting it.  The filters will be added
        via the :mod:`insights.specs.Specs.cloud_cfg` Spec.  When the
        filters is empty, nothing will be parsed.

    The typical content of this file after filtering is (still in Yaml format)::

        debug:
          output: /var/log/cloud-init-debug.log
          verbose: true
        network:
          config:
          - name: eth0
            subnets:
            - type: dhcp
            - type: dhcp6
            type: physical
          version: 1
        ssh_deletekeys: 1

    Attributes:
        data(dict): Cloud-init network configuration.

    Examples:
        >>> from insights.core.filters import add_filter
        >>> from insights.specs import Specs
        >>> add_filter(Specs.cloud_cfg, ['network', 'debug', 'ssh_deletekeys'])
        >>> 'users' not in cloud_cfg
        True
        >>> cloud_cfg['network']['version'] == 1
        True
        >>> cloud_cfg['network']['config'] == [{"type": "physical", "name": "eth0", "subnets": [{"type": "dhcp"}, {"type": "dhcp6"}]}]
        True
    """
    pass



from insights.parsers.nova_log import NovaApiLog, NovaComputeLog
from insights.tests import context_wrap

from datetime import datetime

api_log = """
2016-08-12 13:15:46.343 32386 WARNING nova.api.ec2.cloud [-] Deprecated: The in tree EC2 API is deprecated as of Kilo release and may be removed in a future release. The stackforge ec2-api project http://git.openstack.org/cgit/stackforge/ec2-api/ is the target replacement for this functionality.
2016-08-12 14:27:16.498 32499 WARNING keystonemiddleware.auth_token [-] Authorization failed for token
2016-08-12 14:27:16.500 32499 WARNING keystonemiddleware.auth_token [-] Identity response: {"error": {"message": "Could not find token: 786c3cee52d14baeae98262c6a2f3a4e", "code": 404, "title": "Not Found"}}
2016-08-12 14:27:16.502 32499 WARNING keystonemiddleware.auth_token [-] Authorization failed for token
"""""


def test_nova_api_log():
    log = NovaApiLog(context_wrap(api_log))
    assert len(log.get(["WARNING", "Authorization failed"])) == 2
    assert len(log.get("786c3cee52d14baeae98262c6a2f3a4e")) == 1
    assert len(log.get("Authorization failed")) == 2
    assert len(list(log.get_after(datetime(2016, 8, 12, 14, 20, 0)))) == 3


def test_nova_compute_log():
    # use the same log file to do test. That is ok.
    log = NovaComputeLog(context_wrap(api_log))
    assert len(log.get(["WARNING", "Authorization failed"])) == 2
    assert len(log.get("786c3cee52d14baeae98262c6a2f3a4e")) == 1
    assert len(log.get("Authorization failed")) == 2
    assert len(list(log.get_after(datetime(2016, 8, 12, 14, 20, 0)))) == 3



from insights.combiners.rsyslog_confs import RsyslogAllConf
from insights.core.exceptions import SkipComponent
from insights.parsers.rsyslog_conf import RsyslogConf
from insights.specs.datasources.rsyslog_confs import rsyslog_errorfile
from insights.tests import context_wrap
import pytest


RSYSLOG_ALL_CONF_1 = """
#$DebugLevel 2
#RSYSLOG_DEBUGLOG=/opt/data/syslog/tls_logging.log

# For more information see /usr/share/doc/rsyslog-*/rsyslog_conf.html
# If you experience problems, see http://www.rsyslog.com/doc/troubleshoot.html

#### MODULES ####

# Provides UDP syslog reception
#$ModLoad imudp
#$UDPServerRun 514
$ModLoad imtcp.so
$ModLoad imjournal
$InputTCPMaxSessions 10
$IncludeConfig /etc/rsyslog.d/*.conf
if ($.ev_index != "") then {
    if (lookup("es-debug-selector", $.ev_index) == "on") then
    reset $.random = random(300);
    if ($.random >= 0 and $.random < 100) then {
        action(errorfile="/var/log/rsyslog/es-errors1.log")
    }
    else if ($.random >= 100 and $.random < 200) then {
        action(errorfile="/var/log/rsyslog/es-errors2.log")
    }
    else {
        action(errorfile="/var/log/rsyslog/es-errors3.log")
    }
}

# Provides TCP syslog reception
#$ModLoad imtcp
#$InputTCPServerRun 6514

#Configuration for certificte authentication
$DefaultNetstreamDriver gtls
"""

RSYSLOG_ALL_CONF_2 = """
$WorkDirectory /tmp
*.info -/var/log/test.log
"""

RSYSLOG_ALL_CONF_3 = """
module(load="impstats"
)
global(
    # where to store housekeeping data (stat files, queue files, etc)
    oversizemsg.errorfile="/var/log/oversized.log"
)
module(load="imjournal" PersistStateInterval="1000" StateFile="/var/log/imjournal.state" Ratelimit.Interval="1" Ratelimit.Burst="100000")
module(load="imfile" readTimeout="10")
module(load="mmnormalize")
ruleset(name="setvars"){
    set $!ews-trusted!timestamp2=exec_template("timestamp");
}
ruleset(name="set-st"){
    if $!ews-trusted!sourcetype == '' then {
    }
}
ruleset(name="ews-send"){
    action(name="ews-send"
        template="json-fwd")
}
# BUT  because having the most current data is the most important, we use UDP so that if there is a problem, we dont have current data stuck behind old data (we would rather miss some data than be looking at old data)
ruleset(name="pstats" queue.size="2000" queue.type="FixedArray"){
   action(name="pri-send"
        template="json-fwd")
}
"""


def test_rsyslog_errorfiles_1():
    rsyslog1 = RsyslogConf(context_wrap(RSYSLOG_ALL_CONF_1, path="/etc/rsyslog.conf"))
    rsyslog2 = RsyslogConf(context_wrap(RSYSLOG_ALL_CONF_2, path="/etc/rsyslog.d/test.conf"))
    result = RsyslogAllConf([rsyslog1, rsyslog2])
    broker = {
        RsyslogAllConf: result
    }
    result = rsyslog_errorfile(broker)
    assert result is not None
    assert result == '/var/log/rsyslog/es-errors1.log /var/log/rsyslog/es-errors2.log /var/log/rsyslog/es-errors3.log'


def test_rsyslog_errorfiles_2():
    rsyslog = RsyslogConf(context_wrap(RSYSLOG_ALL_CONF_2, path="/etc/rsyslog.d/test.conf"))
    result = RsyslogAllConf([rsyslog])

    broker = {
        RsyslogAllConf: result
    }
    with pytest.raises(SkipComponent):
        rsyslog_errorfile(broker)


def test_rsyslog_errorfiles_3():
    rsyslog = RsyslogConf(context_wrap(RSYSLOG_ALL_CONF_3, path="/etc/rsyslog.d/test.conf"))
    result = RsyslogAllConf([rsyslog])

    broker = {
        RsyslogAllConf: result
    }
    with pytest.raises(SkipComponent):
        rsyslog_errorfile(broker)



from insights.parsers.blkid import BlockIDInfo
from insights.tests import context_wrap

BLKID_INFO = """
/dev/sda1: UUID="3676157d-f2f5-465c-a4c3-3c2a52c8d3f4" TYPE="xfs"
/dev/sda2: UUID="UVTk76-UWOc-vk7s-galL-dxIP-4UXO-0jG4MH" TYPE="LVM2_member"
/dev/mapper/rhel_hp--dl160g8--3-root: UUID="11124c1d-990b-4277-9f74-c5a34eb2cd04" TYPE="xfs"
/dev/mapper/rhel_hp--dl160g8--3-swap: UUID="c7c45f2d-1d1b-4cf0-9d51-e2b0046682f8" TYPE="swap"
/dev/mapper/rhel_hp--dl160g8--3-home: UUID="c7116820-f2de-4aee-8ea6-0b23c6491598" TYPE="xfs"
/dev/mapper/rhel_hp--dl160g8--3-lv_test: UUID="d403bcbd-0eea-4bff-95b9-2237740f5c8b" TYPE="ext4"
/dev/cciss/c0d1p3: LABEL="/u02" UUID="004d0ca3-373f-4d44-a085-c19c47da8b5e" TYPE="ext3"
/dev/loop0: LABEL="Satellite-5.6.0 x86_64 Disc 0" TYPE="iso9660"
/dev/block/253:1: UUID="f8508c37-eeb1-4598-b084-5364d489031f" TYPE="ext2"
""".strip()

EXPECTED_RESULTS = [{'NAME': "/dev/sda1",
                     'UUID': "3676157d-f2f5-465c-a4c3-3c2a52c8d3f4",
                     'TYPE': "xfs"},
                    {'NAME': "/dev/sda2",
                     'UUID': "UVTk76-UWOc-vk7s-galL-dxIP-4UXO-0jG4MH",
                     'TYPE': "LVM2_member"},
                    {'NAME': "/dev/mapper/rhel_hp--dl160g8--3-root",
                     'UUID': "11124c1d-990b-4277-9f74-c5a34eb2cd04",
                     'TYPE': "xfs"},
                    {'NAME': "/dev/mapper/rhel_hp--dl160g8--3-swap",
                     'UUID': "c7c45f2d-1d1b-4cf0-9d51-e2b0046682f8",
                     'TYPE': "swap"},
                    {'NAME': "/dev/mapper/rhel_hp--dl160g8--3-home",
                     'UUID': "c7116820-f2de-4aee-8ea6-0b23c6491598",
                     'TYPE': "xfs"},
                    {'NAME': "/dev/mapper/rhel_hp--dl160g8--3-lv_test",
                     'UUID': "d403bcbd-0eea-4bff-95b9-2237740f5c8b",
                     'TYPE': "ext4"},
                    {'NAME': "/dev/block/253:1",
                     'UUID': "f8508c37-eeb1-4598-b084-5364d489031f",
                     'TYPE': "ext2"},
                    {'NAME': "/dev/cciss/c0d1p3",
                     'LABEL': "/u02",
                     'UUID': "004d0ca3-373f-4d44-a085-c19c47da8b5e",
                     'TYPE': "ext3"},
                    {'NAME': "/dev/loop0",
                     'LABEL': "Satellite-5.6.0 x86_64 Disc 0",
                     'TYPE': "iso9660"}]


class TestBLKID():
    def test_get_blkid_info(self):
        blkid_info = BlockIDInfo(context_wrap(BLKID_INFO))
        expected_list = dict((r['NAME'], r) for r in EXPECTED_RESULTS)
        assert len(blkid_info.data) == 9

        for result in blkid_info.data:
            assert result == expected_list[result['NAME']]
        ext4_only = blkid_info.filter_by_type("ext4")
        assert len(ext4_only) == 1
        assert ext4_only[0] == {'NAME': "/dev/mapper/rhel_hp--dl160g8--3-lv_test",
                                'UUID': "d403bcbd-0eea-4bff-95b9-2237740f5c8b",
                                'TYPE': "ext4"}
        xfs_only = blkid_info.filter_by_type("xfs")
        expected_list = dict((r['NAME'], r) for r in EXPECTED_RESULTS if r['TYPE'] == "xfs")
        assert len(xfs_only) == 3
        for result in xfs_only:
            assert result == expected_list[result['NAME']]
        ext2_only = blkid_info.filter_by_type("ext2")
        ext2_only[0]["NAME"] == "/dev/block/253:1"



import doctest
import pytest

from datetime import datetime

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import date
from insights.parsers.date import Date, DateUTC, DateParseException, TimeDateCtlStatus
from insights.tests import context_wrap


DATE_OUTPUT1 = "Mon May 30 10:49:14 %s 2016"
DATE_OUTPUT2 = "Thu Oct 22 12:59:28 %s 2015"
DATE_OUTPUT_TRUNCATED = "Thu Oct 22"
DATE_OUTPUT_INVALID = "Fon Pla 34 27:63:89 CST 20-1"

TIMEDATECTL_CONTENT1 = """
               Local time: Mon 2022-11-14 22:37:01 EST
           Universal time: Tue 2022-11-15 03:37:01 UTC
                 RTC time: Tue 2022-11-15 03:37:40
                Time zone: America/New_York (EST, -0500)
System clock synchronized: yes
              NTP service: active
          RTC in local TZ: no
"""

TIMEDATECTL_CONTENT2 = """
               Local time: Mon 2022-11-14 23:04:06 PST
           Universal time: Tue 2022-11-15 07:04:06 UTC
                 RTC time: Tue 2022-11-15 07:04:05
                Time zone: US/Pacific (PST, -0800)
System clock synchronized: yes
              NTP service: active
          RTC in local TZ: yes

Warning: The system is configured to read the RTC time in the local time zone.
         This mode cannot be fully supported. It will create various problems
         with time zone changes and daylight saving time adjustments. The RTC
         time is never updated, it relies on external facilities to maintain it.
         If at all possible, use RTC in UTC by calling
         'timedatectl set-local-rtc 0'.
"""

TIMEDATECTL_CONTENT3 = """
      Local time: Mon 2022-11-14 02:33:36 EST
  Universal time: Mon 2022-11-14 07:33:36 UTC
        RTC time: Mon 2022-11-14 07:33:34
       Time zone: America/New_York (EST, -0500)
     NTP enabled: yes
NTP synchronized: yes
 RTC in local TZ: no
      DST active: no
 Last DST change: DST ended at
                  Sun 2022-11-06 01:59:59 EDT
                  Sun 2022-11-06 01:00:00 EST
 Next DST change: DST begins (the clock jumps one hour forward) at
                  Sun 2023-03-12 01:59:59 EST
                  Sun 2023-03-12 03:00:00 EDT
"""

TIMEDATECTL_CONTENT_WRONG_DATE_FORMAT = """
      Local time: Mon 2022-11-14 02:33:36 EST
  Universal time: Mon 2022-11-14 07:33:36 UTC
        RTC time: 2022-11-14 07:33:34
       Time zone: America/New_York (EST, -0500)
     NTP enabled: yes
NTP synchronized: yes
 RTC in local TZ: no
      DST active: no
 Last DST change: DST ended at
                  Sun 2022-11-06 01:59:59 EDT
                  Sun 2022-11-06 01:00:00 EST
 Next DST change: DST begins (the clock jumps one hour forward) at
                  Sun 2023-03-12 01:59:59 EST
                  Sun 2023-03-12 03:00:00 EDT
"""

TIMEDATECTL_CONTENT_NOT_COLON_ALIGNED = """
      Local time: Mon 2022-11-14 02:33:36 EST
  Universal time  : Mon 2022-11-14 07:33:36 UTC
        RTC time: 2022-11-14 07:33:34
       Time zone: America/New_York (EST, -0500)
     NTP enabled: yes
NTP synchronized: yes
 RTC in local TZ: no
      DST active: no
 Last DST change: DST ended at
                  Sun 2022-11-06 01:59:59 EDT
                  Sun 2022-11-06 01:00:00 EST
 Next DST change: DST begins (the clock jumps one hour forward) at
                  Sun 2023-03-12 01:59:59 EST
                  Sun 2023-03-12 03:00:00 EDT
"""

TIMEDATECTL_CONTENT4_WITHOUT_INFO = """"""

TIMEDATECTL_CONTENT4_WITHOUT_COLON_OUTPUT = """
this is just test
""".strip()


def test_get_date1():
    DATE = DATE_OUTPUT1 % ('CST')
    date_info = Date(context_wrap(DATE))
    assert date_info.data == DATE
    assert date_info.datetime is not None
    assert date_info.timezone == 'CST'


def test_get_date2():
    DATE = DATE_OUTPUT2 % ('EDT')
    date_info = Date(context_wrap(DATE))
    assert date_info.data == DATE
    assert date_info.datetime is not None
    assert date_info.timezone == 'EDT'


def test_get_date_truncated():
    with pytest.raises(DateParseException) as e_info:
        Date(context_wrap(DATE_OUTPUT_TRUNCATED))
    assert 'Expected six date parts.  Got ' in str(e_info.value)


def test_get_date_invalid():
    with pytest.raises(DateParseException) as e_info:
        Date(context_wrap(DATE_OUTPUT_INVALID))
    assert DATE_OUTPUT_INVALID in str(e_info.value)


def test_get_date3():
    DATE = DATE_OUTPUT1 % ('UTC')
    date_info = DateUTC(context_wrap(DATE))
    assert date_info.data == DATE
    assert date_info.datetime is not None
    assert date_info.timezone == 'UTC'


def test_timedatectl():
    timectl1 = TimeDateCtlStatus(context_wrap(TIMEDATECTL_CONTENT1, strip=False))
    assert timectl1 is not None
    assert 'local_time' in timectl1
    local_time_val = datetime.strptime('2022-11-14 22:37:01', '%Y-%m-%d %H:%M:%S')
    assert timectl1['local_time'] == local_time_val
    assert 'universal_time' in timectl1
    universal_time_val = datetime.strptime('2022-11-15 03:37:01', '%Y-%m-%d %H:%M:%S')
    assert timectl1['universal_time'] == universal_time_val
    assert 'rtc_time' in timectl1
    rtc_time_val = datetime.strptime('2022-11-15 03:37:40', '%Y-%m-%d %H:%M:%S')
    assert timectl1['rtc_time'] == rtc_time_val
    assert timectl1['system_clock_synchronized'] == 'yes'
    assert timectl1['rtc_in_local_tz'] == 'no'
    assert timectl1['ntp_service'] == 'active'

    timectl2 = TimeDateCtlStatus(context_wrap(TIMEDATECTL_CONTENT2, strip=False))
    assert 'warning' in timectl2
    assert 'timedatectl set-local-rtc 0' in timectl2['warning']

    timectl3 = TimeDateCtlStatus(context_wrap(TIMEDATECTL_CONTENT3, strip=False))
    assert 'last_dst_change' in timectl3
    assert timectl3['system_clock_synchronized'] == 'yes'
    assert 'DST ended at Sun 2022-11-06 01:59:59 EDT Sun 2022-11-06 01:00:00 EST' in timectl3['last_dst_change']
    assert 'DST begins (the clock jumps one hour forward) at Sun 2023-03-12 01:59:59 EST Sun 2023-03-12 03:00:00 EDT' in timectl3['next_dst_change']


def test_timedatectl_except():
    with pytest.raises(SkipComponent):
        TimeDateCtlStatus(context_wrap(TIMEDATECTL_CONTENT4_WITHOUT_INFO, strip=False))
    with pytest.raises(ParseException):
        TimeDateCtlStatus(context_wrap(TIMEDATECTL_CONTENT4_WITHOUT_COLON_OUTPUT, strip=False))
    with pytest.raises(DateParseException):
        TimeDateCtlStatus(context_wrap(TIMEDATECTL_CONTENT_WRONG_DATE_FORMAT, strip=False))
    with pytest.raises(ParseException):
        TimeDateCtlStatus(context_wrap(TIMEDATECTL_CONTENT_NOT_COLON_ALIGNED, strip=False))


def test_doc():
    failed_count, _ = doctest.testmod(
        date, globs={'ctl_info': TimeDateCtlStatus(context_wrap(TIMEDATECTL_CONTENT2, strip=False))}
    )
    assert failed_count == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import imagemagick_policy
from insights.parsers.imagemagick_policy import ImageMagickPolicy
from insights.tests import context_wrap

XML_POLICY_COMMENTED_SOME = """
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE policymap [
  <!ELEMENT policymap (policy)+>
  <!ATTLIST policymap xmlns CDATA #FIXED ''>
  <!ELEMENT policy EMPTY>
  <!ATTLIST policy xmlns CDATA #FIXED '' domain NMTOKEN #REQUIRED
    name NMTOKEN #IMPLIED pattern CDATA #IMPLIED rights NMTOKEN #IMPLIED
    stealth NMTOKEN #IMPLIED value CDATA #IMPLIED>
]>
<!--
  Configure ImageMagick policies.
  ...
-->
<policymap>
  <!-- <policy domain="system" name="shred" value="2"/> -->
  <!-- <policy domain="system" name="precision" value="6"/> -->
  <!-- <policy domain="system" name="memory-map" value="anonymous"/> -->
  <policy domain="coder" rights="none" pattern="EPHEMERAL"/>
  <policy domain="coder" rights="none" pattern="HTTPS"/>
  <policy domain="coder" rights="none" pattern="HTTP"/>
  <policy domain="coder" rights="none" pattern="URL"/>
  <policy domain="coder" rights="none" pattern="FTP"/>
  <policy domain="coder" rights="none" pattern="MVG"/>
  <policy domain="coder" rights="none" pattern="MSL"/>
  <policy domain="coder" rights="none" pattern="TEXT" />
  <policy domain="coder" rights="none" pattern="LABEL"/>
  <policy domain="path" rights="none" pattern="@*"/>
</policymap>
"""

XML_POLICY_COMMENTED_ALL = """
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE policymap [
  <!ELEMENT policymap (policy)+>
  <!ATTLIST policymap xmlns CDATA #FIXED ''>
  <!ELEMENT policy EMPTY>
  <!ATTLIST policy xmlns CDATA #FIXED '' domain NMTOKEN #REQUIRED
    name NMTOKEN #IMPLIED pattern CDATA #IMPLIED rights NMTOKEN #IMPLIED
    stealth NMTOKEN #IMPLIED value CDATA #IMPLIED>
]>
<!--
  Configure ImageMagick policies.
  ...
-->
<policymap>
  <!-- <policy domain="resource" name="temporary-path" value="/tmp"/> -->
  <!-- <policy domain="resource" name="memory" value="2GiB"/> -->
  <!-- <policy domain="resource" name="map" value="4GiB"/> -->
  <!-- <policy domain="resource" name="width" value="10KP"/> -->
  <!-- <policy domain="resource" name="height" value="10KP"/> -->
</policymap>
"""

XML_EMPTY = """
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE policymap [
  <!ELEMENT policymap (policy)+>
  <!ATTLIST policymap xmlns CDATA #FIXED ''>
  <!ELEMENT policy EMPTY>
  <!ATTLIST policy xmlns CDATA #FIXED '' domain NMTOKEN #REQUIRED
    name NMTOKEN #IMPLIED pattern CDATA #IMPLIED rights NMTOKEN #IMPLIED
    stealth NMTOKEN #IMPLIED value CDATA #IMPLIED>
]>
<!--
  Configure ImageMagick policies.
  ...
-->
<!--
  <policymap>
    <policy domain="coder" rights="none" pattern="EPHEMERAL"/>
    <policy domain="coder" rights="none" pattern="HTTPS"/>
    <policy domain="coder" rights="none" pattern="HTTP"/>
    <policy domain="coder" rights="none" pattern="URL"/>
    <policy domain="coder" rights="none" pattern="FTP"/>
  </policymap>
-->
"""

TEST_CASES = [
    (XML_POLICY_COMMENTED_SOME, 10),
    (XML_POLICY_COMMENTED_ALL, 0),
    (XML_EMPTY, 0),
]


def test_no_data():
    with pytest.raises(SkipComponent):
        ImageMagickPolicy(context_wrap(""))


@pytest.mark.parametrize("output, length", TEST_CASES)
def test_parsing_policymap(output, length):
    xml = ImageMagickPolicy(context_wrap(output))
    assert len(xml.policies) == length


def test_doc_examples():
    env = {
        "imagemagick_policy": ImageMagickPolicy(context_wrap(XML_POLICY_COMMENTED_SOME))
    }
    failed, total = doctest.testmod(imagemagick_policy, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import azure_instance_plan
from insights.parsers.azure_instance_plan import AzureInstancePlan
from insights.tests import context_wrap

AZURE_PLAN_1 = '{"name": "rhel7", "product": "rhel", "publisher": "Red Hat"}'
AZURE_PLAN_2 = '{"name": "", "product": "", "publisher": "Red Hat"}'
AZURE_PLAN_3 = '{"name": "", "product": "", "publisher": ""}'

AZURE_PLAN_4 = """
 % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                  Dload  Upload   Total   Spent    Left  Speed
   0     0    0     0    0     0      0      0 --:--:-- --:--:-- --:--:--     0
 100  1126  100  1126    0     0  1374k      0 --:--:-- --:--:-- --:--:-- 1099k
{"name": "rhel7", "product": "rhel", "publisher": "Red Hat"}
"""

AZURE_PLAN_DOC = '{"name": "planName", "product": "planProduct", "publisher": "planPublisher"}'

AZURE_PLAN_AB_1 = """
curl: (7) Failed to connect to 169.254.169.254 port 80: Connection timed out
""".strip()
AZURE_PLAN_AB_2 = """
curl: (7) couldn't connect to host
""".strip()
AZURE_PLAN_AB_3 = """
curl: (28) connect() timed out!
""".strip()


def test_azure_instance_place_ab_other():
    with pytest.raises(SkipComponent):
        AzureInstancePlan(context_wrap(AZURE_PLAN_AB_1))

    with pytest.raises(SkipComponent):
        AzureInstancePlan(context_wrap(AZURE_PLAN_AB_2))

    with pytest.raises(SkipComponent):
        AzureInstancePlan(context_wrap(AZURE_PLAN_AB_3))

    with pytest.raises(SkipComponent):
        AzureInstancePlan(context_wrap(''))

    with pytest.raises(ParseException):
        AzureInstancePlan(context_wrap(AZURE_PLAN_4))


def test_azure_instance_plan():
    azure = AzureInstancePlan(context_wrap(AZURE_PLAN_1))
    assert azure.name == "rhel7"
    assert azure.product == "rhel"
    assert azure.publisher == "Red Hat"
    assert azure.raw == '{"name": "rhel7", "product": "rhel", "publisher": "Red Hat"}'

    azure = AzureInstancePlan(context_wrap(AZURE_PLAN_2))
    assert azure.name is None
    assert azure.product is None
    assert azure.publisher == "Red Hat"
    assert azure.raw == '{"name": "", "product": "", "publisher": "Red Hat"}'

    azure = AzureInstancePlan(context_wrap(AZURE_PLAN_3))
    assert azure.name is None
    assert azure.product is None
    assert azure.publisher is None
    assert azure.raw == '{"name": "", "product": "", "publisher": ""}'


def test_doc_examples():
    env = {
        'azure_plan': AzureInstancePlan(context_wrap(AZURE_PLAN_DOC))
    }
    failed, total = doctest.testmod(azure_instance_plan, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import ContentException, ParseException, SkipComponent
from insights.parsers import corosync_cmapctl
from insights.tests import context_wrap


COROSYNC_CONTENT_1 = """
config.totemconfig_reload_in_progress (u8) = 0
internal_configuration.service.0.name (str) = corosync_cmap
internal_configuration.service.0.ver (u32) = 0
internal_configuration.service.1.name (str) = corosync_cfg
internal_configuration.service.1.ver (u32) = 0
internal_configuration.service.2.name (str) = corosync_cpg
internal_configuration.service.2.ver (u32) = 0
internal_configuration.service.3.name (str) = corosync_quorum
internal_configuration.service.3.ver (u32) = 0
internal_configuration.service.4.name (str) = corosync_pload
internal_configuration.service.4.ver (u32) = 0
internal_configuration.service.5.name (str) = corosync_votequorum
internal_configuration.service.5.ver (u32) = 0
logging.logfile (str) = /var/log/cluster/corosync.log
logging.to_logfile (str) = yes
logging.to_syslog (str) = yes
nodelist.local_node_pos (u32) = 1
nodelist.node.0.nodeid (u32) = 1
""".strip()

COROSYNC_CONTENT_2 = """
stats.schedmiss.0.delay (flt) = 2023.957031
stats.schedmiss.0.timestamp (u64) = 5106558848098
stats.schedmiss.1.delay (flt) = 2023.436279
stats.schedmiss.1.timestamp (u64) = 5106556824141
stats.schedmiss.2.delay (flt) = 2030.076904
stats.schedmiss.2.timestamp (u64) = 5106554800704
stats.schedmiss.3.delay (flt) = 2022.936890
stats.schedmiss.3.timestamp (u64) = 5106552770627
stats.schedmiss.4.delay (flt) = 2024.749023
stats.schedmiss.4.timestamp (u64) = 5106550747691
stats.schedmiss.5.delay (flt) = 2021.841553
stats.schedmiss.5.timestamp (u64) = 5106548722942
stats.schedmiss.6.delay (flt) = 2024.137207
stats.schedmiss.6.timestamp (u64) = 5106546701100
stats.schedmiss.7.delay (flt) = 2021.287476
stats.schedmiss.7.timestamp (u64) = 5106544676963
stats.schedmiss.8.delay (flt) = 2025.073853
stats.schedmiss.8.timestamp (u64) = 5106542655675
stats.schedmiss.9.delay (flt) = 2473.122070
stats.schedmiss.9.timestamp (u64) = 5106540630601
""".strip()

COROSYNC_CONTENT_3 = """
""".strip()

COROSYNC_CONTENT_4 = """
corosync-cmapctl: invalid option -- 'C'
""".strip()

COROSYNC_CONTENT_5 = """
stats.schedmiss.7.timestamp (u64)5106544676963
stats.schedmiss.8.delay (flt) = 2025.073853
stats.schedmiss.8.timestamp (u64) = 5106542655675
stats.schedmiss.9.delay (flt) = 2473.122070
stats.schedmiss.9.timestamp (u64) = 5106540630601
""".strip()

COROSYNC_CONTENT_6 = """
config.totemconfig_reload_in_progress (u8) = 0
internal_configuration.service.0.name (str) = corosync_cmap
internal_configuration.service.0.ver (u32) = 0
internal_configuration.service.1.name (str) = corosync_cfg
internal_configuration.service.1.ver (u32) = 0
internal_configuration.service.2.name (str) = corosync_cpg
internal_configuration.service.2.ver (u32) = 0
internal_configuration.service.3.name (str) = corosync_quorum
internal_configuration.service.3.ver (u32) = 0
runtime.schedmiss.delay (flt) = 2282.403320
runtime.schedmiss.timestamp (u64) = 1589895874915
""".strip()


def test_corosync_doc_examples():
    env = {
        'corosync': corosync_cmapctl.CorosyncCmapctl(context_wrap(COROSYNC_CONTENT_1, path='corosync_cmpctl')),
    }
    failed, total = doctest.testmod(corosync_cmapctl, globs=env)
    assert failed == 0


def test_state_schemiss():
    corodata = corosync_cmapctl.CorosyncCmapctl(context_wrap(COROSYNC_CONTENT_2, path='corosync-cmapctl_-m_stats_stats.schedmiss'))
    assert 'stats.schedmiss.0.delay' in corodata
    assert corodata['stats.schedmiss.0.delay'] == '2023.957031'
    assert corodata['stats.schedmiss.0.timestamp'] == '5106558848098'
    assert len(corodata) == 20


def test_exception():
    with pytest.raises(SkipComponent):
        corosync_cmapctl.CorosyncCmapctl(context_wrap(COROSYNC_CONTENT_3, path="corosync_cmpctl"))
    with pytest.raises(ContentException):
        corosync_cmapctl.CorosyncCmapctl(context_wrap(COROSYNC_CONTENT_4, path="corosync_cmpctl_-C"))
    with pytest.raises(ParseException):
        corosync_cmapctl.CorosyncCmapctl(context_wrap(COROSYNC_CONTENT_5, path="corosync_cmpctl"))


def test_runtime_schemiss():
    corodata = corosync_cmapctl.CorosyncCmapctl(context_wrap(COROSYNC_CONTENT_6, path='corosync-cmapctl'))
    assert "runtime.schedmiss.delay" in corodata
    assert corodata['runtime.schedmiss.delay'] == '2282.403320'
    assert "runtime.schedmiss.timestamp" in corodata
    assert corodata['runtime.schedmiss.timestamp'] == '1589895874915'
